import os
from pathlib import Path
import re
import sys
import base64

from loguru import logger
import tomli as tomllib
from schema import And, Optional, Or, Regex, Schema, SchemaError
from appdirs import user_data_dir

from . import __name__ as __product__
from .var import console


async def convert_session(accounts, basedir=None):
    from .telechecker.tele import ClientsSession

    results = []
    for a in accounts:
        async with ClientsSession.from_config({"telegram": [a], "basedir": basedir}) as clients:
            async for tg in clients:
                session_string = await tg.export_session_string()
                results.append({**a, "session": session_string})
    return results


def check_config(config):
    """验证配置文件格式"""
    PositiveInt = lambda: And(int, lambda n: n > 0)
    schema = Schema(
        {
            Optional("time"): str,
            Optional("watchtime"): str,
            Optional("listentime"): str,
            Optional("interval"): Or(PositiveInt(), str),
            Optional("timeout"): PositiveInt(),
            Optional("retries"): PositiveInt(),
            Optional("concurrent"): PositiveInt(),
            Optional("watch_concurrent"): int,
            Optional("listen_concurrent"): int,
            Optional("random"): PositiveInt(),
            Optional("notifier"): Or(str, bool, int),
            Optional("notify_immediately"): bool,
            Optional("nofail"): bool,
            Optional("proxy"): Schema(
                {
                    Optional("hostname"): Regex(
                        r"^(?=.{1,255}$)[0-9A-Za-z](?:(?:[0-9A-Za-z]|-){0,61}[0-9A-Za-z])?(?:\.[0-9A-Za-z](?:(?:[0-9A-Za-z]|-){0,61}[0-9A-Za-z])?)*\.?$"
                    ),
                    Optional("port"): And(int, lambda n: n > 1024 and n < 65536),
                    Optional("scheme"): Schema(Or("socks5", "http")),
                    Optional("username"): str,
                    Optional("password"): str,
                }
            ),
            Optional("service"): Schema(
                {
                    Optional("checkiner"): [str],
                    Optional("monitor"): [str],
                    Optional("messager"): [str],
                }
            ),
            Optional("telegram"): [
                Schema(
                    {
                        "phone": str,
                        Optional("checkin"): bool,
                        Optional("monitor"): bool,
                        Optional("send"): bool,
                        Optional("api_id"): Regex(r"^\d+$"),
                        Optional("api_hash"): Regex(r"^[a-z0-9]+$"),
                        Optional("session"): str,
                        Optional("service"): Schema(
                            {
                                Optional("checkiner"): [str],
                                Optional("monitor"): [str],
                                Optional("messager"): [str],
                            }
                        ),
                    }
                )
            ],
            Optional("emby"): [
                Schema(
                    {
                        "url": Regex(
                            r"(http|https):\/\/([\w_-]+(?:(?:\.[\w_-]+)+))([\w.,@?^=%&:\/~+#-]*[\w@?^=%&\/~+#-])"
                        ),
                        "username": str,
                        "password": str,
                        Optional("time"): Or(int, [PositiveInt()]),
                        Optional("continuous"): bool,
                        Optional("jellyfin"): bool,
                        Optional("ua"): str,
                        Optional("client"): str,
                        Optional("device"): str,
                        Optional("device_id"): str,
                        Optional("allow_multiple"): bool,
                        Optional("allow_stream"): bool,
                        Optional("cf_challenge"): bool,
                    }
                )
            ],
            Optional("subsonic"): [
                Schema(
                    {
                        "url": Regex(
                            r"(http|https):\/\/([\w_-]+(?:(?:\.[\w_-]+)+))([\w.,@?^=%&:\/~+#-]*[\w@?^=%&\/~+#-])"
                        ),
                        "username": str,
                        "password": str,
                        Optional("time"): Or(int, [PositiveInt()]),
                        Optional("ua"): str,
                        Optional("client"): str,
                        Optional("version"): str,
                    }
                )
            ],
            Optional("checkiner"): Schema({str: Schema({}, ignore_extra_keys=True)}),
            Optional("monitor"): Schema({str: Schema({}, ignore_extra_keys=True)}),
            Optional("messager"): Schema({str: Schema({}, ignore_extra_keys=True)}),
        }
    )
    try:
        schema.validate(config)

    except SchemaError as e:
        return e
    else:
        return None


def write_faked_config(path, quiet=False):
    """生成配置文件骨架, 并填入生成的信息."""
    from tomlkit import document, nl, comment, item, dump
    from faker import Faker
    from faker.providers import internet, profile

    from .telechecker.main import get_names
    from . import __version__, __url__

    if not quiet:
        logger.warning("需要输入一个toml格式的config文件.")
        logger.warning(f'您可以根据生成的参考配置文件 "{path}" 进行配置')

    fake = Faker()
    fake.add_provider(internet)
    fake.add_provider(profile)

    doc = document()
    doc.add(comment("这是一个配置文件范例."))
    doc.add(comment("所有账户信息为生成, 请填写您的账户信息."))
    doc.add(comment(f"查看帮助与详情: {__url__}#安装与使用"))
    doc.add(nl())
    doc.add(comment('每天进行 Telegram Bot 签到的时间范围, 等同于命令行 "-c" 参数.'))
    doc["time"] = "<8:00AM,10:00AM>"
    doc.add(nl())
    doc.add(comment('每隔几天进行 Emby 保活, 或随机范围, 等同于命令行 "-e" 参数.'))
    doc["interval"] = "<3,12>"
    doc.add(nl())
    doc.add(comment("Emby 保活开始时间的时间范围."))
    doc["watchtime"] = "<11:00AM,11:00PM>"
    doc.add(nl())
    doc.add(comment("将关键信息发送到第一个 Telegram 账号, 设为 N 以发送到第 N 个."))
    doc["notifier"] = True
    doc.add(nl())
    doc.add(comment("每个 Telegram Bot 签到的最大尝试时间 (秒)."))
    doc["timeout"] = 240
    doc.add(nl())
    doc.add(comment("每个 Telegram Bot 签到的最大尝试次数."))
    doc["retries"] = 4
    doc.add(nl())
    doc.add(comment("最大可同时进行的 Telegram Bot 签到."))
    doc["concurrent"] = 1
    doc.add(nl())
    doc.add(comment("最大可同时进行的 Emby 站点保活."))
    doc["watch_concurrent"] = 3
    doc.add(nl())
    doc.add(comment("计划任务时, 各站点之间签到的随机时间差异 (分钟), 所有站点签到将在该时间范围内完成."))
    doc["random"] = 60
    doc.add(nl())
    doc.add(comment(f"代理设置, Emby 和 Telegram 均将通过此代理连接, 服务器位于国内时请配置代理并取消注释."))
    doc.add(comment(f"详见: https://emby-keeper.github.io/guide/配置文件#proxy-子项"))
    proxy = item(
        {
            "proxy": {
                "hostname": "127.0.0.1",
                "port": 1080,
                "scheme": "socks5",
            }
        }
    )
    proxy["proxy"]["scheme"].comment("可选: http / socks5")
    for line in proxy.as_string().strip().split("\n"):
        doc.add(comment(line))
    doc.add(nl())
    doc.add(comment(f"服务设置, 当您需要禁用某些站点时, 请将该段取消注释并修改."))
    doc.add(comment(f"该部分内容是根据 {__product__.capitalize()} {__version__} 生成的."))
    doc.add(comment(f"详见: https://emby-keeper.github.io/guide/配置文件#service-子项"))
    doc.add(
        comment(
            f'使用 checkiner = ["all"] 以启用所有签到器, 使用 checkiner = ["sgk"] 以启用所有社工库签到器.'
        )
    )
    doc.add(nl())
    service = item(
        {
            "service": {
                "checkiner": get_names("checkiner"),
                "monitor": get_names("monitor"),
                "messager": get_names("messager"),
            }
        }
    )
    doc.add(comment(f"默认启用站点:"))
    for line in service.as_string().strip().split("\n"):
        doc.add(comment(line))
    doc.add(nl())
    service_all = item(
        {
            "service": {
                "checkiner": get_names("checkiner", allow_ignore=True),
                "monitor": get_names("monitor", allow_ignore=True),
                "messager": get_names("messager", allow_ignore=True),
            }
        }
    )
    doc.add(comment(f"全部可用站点:"))
    for line in service_all.as_string().strip().split("\n"):
        doc.add(comment(line))
    doc.add(nl())
    doc.add(comment("Telegram 账号设置, 您可以重复该片段多次以增加多个账号."))
    doc.add(comment(f"详见: https://emby-keeper.github.io/guide/配置文件#telegram-子项"))
    telegram = []
    for _ in range(2):
        t = item(
            {
                "phone": f'+861{fake.numerify(text="##########")}',
                "checkin": True,
                "send": False,
                "monitor": False,
            }
        )
        telegram.append(t)
    doc["telegram"] = telegram
    for t in doc["telegram"]:
        t.value.item("send").comment("启用该账号的自动水群功能 (需要高级账号, 谨慎使用)")
        t.value.item("monitor").comment("启用该账号的自动监控功能 (需要高级账号)")
    doc.add(nl())
    doc.add(comment("Emby 账号设置, 您可以重复该片段多次以增加多个账号."))
    doc.add(comment(f"详见: https://emby-keeper.github.io/guide/配置文件#emby-子项"))
    emby = []
    for _ in range(2):
        t = item(
            {
                "url": fake.url(["https"]),
                "username": fake.profile()["username"],
                "password": fake.password(),
                "time": [120, 240],
            }
        )
        t["time"].comment("模拟观看的时长范围 (秒)")
        emby.append(t)
    doc["emby"] = emby
    if isinstance(path, (str, Path)):
        with open(path, "w+", encoding="utf-8") as f:
            dump(doc, f)
    else:
        dump(doc, path)


async def interactive_config(config: dict = {}, basedir=None):
    """交互式配置生成器."""

    from tomlkit import item
    from rich import get_console
    from rich.prompt import Prompt, IntPrompt, Confirm

    from . import __url__

    pad = " " * 23
    logger.info("我们将为您生成配置, 需要您根据提示填入信息, 并按回车确认.")
    logger.info(f"配置帮助详见: {__url__}.")
    logger.info(f"若需要重新开始, 请点击右上方的刷新按钮.")
    logger.info(f"若您需要更加高级的配置, 请使用右上角的 Config 按钮以修改配置文件.")
    telegrams = config.get("telegram", [])
    while True:
        if len(telegrams) > 0:
            logger.info(
                f'您当前填写了 {len(telegrams)} 个 Telegram 账号信息: {", ".join([t["phone"] for t in telegrams])}'
            )
            more = Confirm.ask(pad + "是否继续添加?", default=False, console=console)
        else:
            more = Confirm.ask(pad + "是否添加 Telegram 账号?", default=True, console=console)
        if not more:
            break
        phone = Prompt.ask(
            pad + "请输入您的 Telegram 账号 (带国家区号) [dark_green](+861xxxxxxxxxx)[/]",
            console=console,
        )
        monitor = Confirm.ask(
            pad + "是否开启该账号的自动监控功能? (需要高级账号)", default=False, console=console
        )
        send = Confirm.ask(
            pad + "是否开启该账号的自动水群功能? (需要高级账号)", default=False, console=console
        )
        telegrams.append({"phone": phone, "send": False, "monitor": monitor, "send": send})
    if telegrams:
        logger.info(f"即将尝试登录各账户并存储凭据, 请耐心等待.")
        telegrams = await convert_session(telegrams, basedir=basedir)
    embies = config.get("emby", [])
    while True:
        if len(embies) > 0:
            logger.info(f"您当前填写了 {len(embies)} 个 Emby 账号信息.")
            more = Confirm.ask(pad + "是否继续添加?", default=False, console=console)
        else:
            more = Confirm.ask(pad + "是否添加 Emby 账号?", default=True, console=console)
        if not more:
            break
        url = Prompt.ask(
            pad + "请输入您的 Emby 站点 URL [dark_green](https://abc.com:443)[/]", console=console
        )
        username = Prompt.ask(pad + "请输入您在该 Emby 站点的用户名", console=console)
        password = Prompt.ask(
            pad + "请输入您在该 Emby 站点的密码 (不显示, 按回车确认)",
            password=True,
            console=console,
        )
        while True:
            time = Prompt.ask(
                pad + "设置模拟观看时长范围 (秒), 用空格分隔",
                default="120 240",
                show_default=True,
                console=console,
            )
            if " " in time:
                try:
                    time = [int(t) for t in time.split(None, 1)]
                    break
                except ValueError:
                    logger.warning(f"时长设置不正确, 请重新输入.")
            else:
                try:
                    time = int(time)
                    break
                except ValueError:
                    logger.warning(f"时长设置不正确, 请重新输入.")
        embies.append({"url": url, "username": username, "password": password, "time": time})
    config = {"telegram": telegrams, "emby": embies}
    advanced = Confirm.ask(pad + "是否配置高级设置", default=False, console=console)
    config.setdefault("notifier", True)
    config.setdefault("timeout", 240)
    config.setdefault("retries", 4)
    config.setdefault("concurrent", 1)
    config.setdefault("random", 15)
    if advanced:
        logger.info("发送关键日志消息到以下哪个账户?")
        logger.info(f"\t0. 不使用消息推送功能")
        for i, t in enumerate(telegrams):
            logger.info(f'\t{i+1}. {t["phone"]}')
        config["notifier"] = IntPrompt.ask(pad + "请选择", default=1, console=console)
        config["timeout"] = IntPrompt.ask(
            pad + "设置每个 Telegram Bot 签到的最大尝试时间 (秒)",
            default=config["timeout"],
            show_default=True,
            console=console,
        )
        config["retries"] = IntPrompt.ask(
            pad + "设置每个 Telegram Bot 签到的最大尝试次数",
            default=config["retries"],
            show_default=True,
            console=console,
        )
        config["concurrent"] = IntPrompt.ask(
            pad + "设置最大可同时进行的 Telegram Bot 签到",
            default=config["concurrent"],
            show_default=True,
            console=console,
        )
        config["random"] = IntPrompt.ask(
            pad + "设置计划任务时, 各站点之间签到的随机时间差异 (分钟)",
            default=config["random"],
            show_default=True,
            console=console,
        )
    content = item(config).as_string().encode()
    content = base64.b64encode(content)
    logger.info(
        f"您的配置[green]已生成完毕[/]! 您需要将以下内容写入托管平台的 EK_CONFIG 环境变量 ([red]SECRET[/]), 否则配置将在重启后丢失."
    )
    print()
    get_console().rule("EK_CONFIG")
    print(content.decode())
    get_console().rule()
    print()
    start_now = Confirm.ask(pad + "是否立即启动?", default=True, console=console)
    if start_now:
        return config


def load_env_config(data: str):
    """从来自环境变量的加密数据读入配置."""

    data = base64.b64decode(re.sub(r"\s+", "", data).encode())
    try:
        config = tomllib.loads(data.decode())
    except (tomllib.TOMLDecodeError, UnicodeDecodeError):
        logger.error("配置格式错误, 请调整并重试.")
        sys.exit(252)
    else:
        logger.debug("您正在使用环境变量配置.")
    return config


async def check_telegram(config: dict, basedir=None):
    from tomlkit import item
    from rich import get_console
    from rich.prompt import Confirm

    pad = " " * 23
    telegrams = []
    for a in config.get("telegram", []):
        if not a.get("session", None):
            telegrams.append(a)
    if telegrams:
        logger.info("即将尝试登陆各个账号.")
        telegrams = await convert_session(telegrams, basedir=basedir)
        config["telegram"] = telegrams
        content = item(config).as_string().encode()
        content = base64.b64encode(content)
        logger.info(
            f"您已登陆到 Telegram! 您需要将以下内容重新写入托管平台的 EK_CONFIG 环境变量 ([red]SECRET[/]), 否则登陆状态将在重启后丢失."
        )
        print()
        get_console().rule("EK_CONFIG")
        print(content.decode())
        get_console().rule()
        print()
        start_now = Confirm.ask(pad + "是否立即启动?", default=True, console=console)
        if start_now:
            return config
    else:
        return config


async def prepare_config(config_file=None, basedir=None, public=False, windows=False):
    """
    准备配置
    参数:
        config_file: 配置文件
        public: 公共服务器模式, 将提示交互式配置生成
    """
    config = {}
    basedir = Path(basedir or user_data_dir(__product__))
    basedir.mkdir(parents=True, exist_ok=True)
    if public:
        logger.info(f'工作目录: "{basedir}"')
    else:
        logger.info(f'工作目录: "{basedir}", 您的用户数据相关文件将存储在此处, 请妥善保管.')
    docker = bool(os.environ.get("EK_IN_DOCKER", False))
    if docker:
        logger.info("当前在 Docker 容器中运行, 请确认该目录已挂载, 否则文件将在容器重建后丢失.")
    env_config = os.environ.get(f"EK_CONFIG", None)
    if env_config:
        config = load_env_config(env_config)
        if public:
            config = await check_telegram(config, basedir=basedir)
            if not config:
                sys.exit(250)
    else:
        if public:
            if config_file:
                try:
                    with open(config_file, "rb") as f:
                        config = tomllib.load(f)
                except tomllib.TOMLDecodeError as e:
                    logger.error(f"TOML 配置文件错误: {e}.")
                    sys.exit(252)
                else:
                    logger.info(f"将以 {Path(config_file).name} 为基础生成配置.")
            config = await interactive_config(config, basedir=basedir)
            if not config:
                sys.exit(250)
        else:
            if windows:
                default_config_file = basedir / "config.toml"
            else:
                default_config_file = Path("config.toml")
            if not config_file:
                if not default_config_file.exists():
                    write_faked_config(default_config_file)
                    sys.exit(250)
                else:
                    config_file = default_config_file
            try:
                if not Path(config_file).exists():
                    logger.error(f'配置文件 "{config_file}" 不存在.')
                    sys.exit(251)
                elif config_file == default_config_file:
                    with open(config_file, "rb") as f:
                        config = tomllib.load(f)
                    if not config:
                        write_faked_config(config_file)
                        sys.exit(250)
                else:
                    with open(config_file, "rb") as f:
                        config = tomllib.load(f)
            except tomllib.TOMLDecodeError as e:
                logger.error(f"TOML 配置文件错误: {e}.")
                sys.exit(252)
    error = check_config(config)
    error_str = str(error)
    if error:
        if "'true' should be instance of 'bool'" in error_str:
            error_str += "\n说明: 您可能使用了 \"true\" 或 'true' 作为是/否的配置, 请去除引号 (xxx = true)."
        if "'false' should be instance of 'bool'" in error_str:
            error_str += (
                "\n说明: 您可能使用了 \"false\" 或 'false' 作为是/否的配置, 请去除引号 (xxx = false)."
            )
        if "should be instance of 'int'" in error_str:
            error_str += f"\n说明: 您可能对需要使用数值的参数设置了字符串, 请去除引号 (xxx = 1234)."
        logger.error(
            f"配置文件错误, 请检查配置文件 (参考: https://emby-keeper.github.io/guide/配置文件):\n{error_str}"
        )
        sys.exit(253)
    proxy: dict = config.get("proxy", None)
    if proxy:
        proxy.setdefault("scheme", "socks5")
        proxy.setdefault("hostname", "127.0.0.1")
        proxy.setdefault("port", 1080)
    config["basedir"] = basedir
    return config



from pyrogram.types import Message
from ._base import Monitor

__ignore__ = True


class TestReplyMonitor(Monitor):
    name = "回复 测试"
    chat_name = "api_group"
    chat_allow_outgoing = True
    chat_keyword = r"回复\s*(\w+)"
    chat_delay = 1

    def chat_reply(self, message: Message, keys):
        return f'接收到: "{keys[0]}"'



import asyncio
from dataclasses import dataclass
import hashlib
import time
from typing import Optional, List, Dict, Any
from urllib.parse import urljoin
import random

import httpx
from loguru import logger

logger = logger.bind(scheme="subsonic")


@dataclass
class ServerPingInfo:
    is_ok: bool = True
    type: str = None
    version: str = None
    server_version: str = None
    open_subsonic: bool = True
    error_code: int = None
    error_message: str = None


class Subsonic:
    """
    An async client for interacting with Subsonic API.
    """

    def __init__(
        self,
        server: str,
        username: str,
        password: str,
        proxy: Optional[Dict] = None,
        ua: Optional[str] = None,
        client: Optional[str] = None,
        version: Optional[str] = None,
    ):
        self.server = server.rstrip("/")
        self.username = username
        self.password = password
        self.proxy = proxy
        self.salt = self._generate_salt()
        self.token = self._generate_token()
        self.version = version or "1.15.0"
        self.client = client or "Stream Music"
        self.ua = ua or "Dart/3.4 (dart:io)"

        self._session = None
        self._session_lock = asyncio.Lock()

    def _generate_salt(self, length: int = 6) -> str:
        """Generate a random salt for authentication"""
        return hashlib.md5(str(time.time()).encode()).hexdigest()[:length]

    def _generate_token(self) -> str:
        """Generate authentication token using password and salt"""
        return hashlib.md5(f"{self.password}{self.salt}".encode()).hexdigest()

    async def _get_session(self) -> httpx.AsyncClient:
        """Get or create an HTTP session"""
        async with self._session_lock:
            if not self._session or self._session.is_closed:
                if self.proxy:
                    proxy = f"{self.proxy['scheme']}://"
                    if self.proxy.get("username"):
                        proxy += f"{self.proxy['username']}:{self.proxy['password']}@"
                    proxy += f"{self.proxy['hostname']}:{self.proxy['port']}"
                else:
                    proxy = None

                timeout = httpx.Timeout(connect=10.0, read=None, write=10.0, pool=10.0)

                headers = {"User-Agent": self.ua}
                self._session = httpx.AsyncClient(
                    http2=True,
                    verify=False,
                    follow_redirects=True,
                    timeout=timeout,
                    proxy=proxy,
                    headers=headers,
                )
                logger.debug("Created new Subsonic session")
            return self._session

    async def _make_request(
        self, endpoint: str, params: Optional[Dict[str, Any]] = None, retries: int = 3
    ) -> Dict:
        """
        Make an async API request to Subsonic server
        """
        base_params = {
            "u": self.username,
            "t": self.token,
            "s": self.salt,
            "c": self.client,
            "v": self.version,
            "f": "json",
        }

        if params:
            base_params.update(params)

        url = urljoin(self.server, f"rest/{endpoint}")

        for i in range(retries):
            try:
                session = await self._get_session()
                response = await session.get(url, params=base_params)
                response.raise_for_status()
                return response.json()["subsonic-response"]
            except httpx.HTTPError as e:
                logger.debug(f'Connection to "{url}" failed, retrying: {e.__class__.__name__}: {e}')
                if i == retries - 1:
                    raise
                await asyncio.sleep(1 * (i + 1))

    async def ping(self):
        """Test connection to server"""
        try:
            response = await self._make_request("ping")
            if response.get("status", None) == "ok":
                return ServerPingInfo(
                    type=response.get("type", None),
                    version=response.get("version", None),
                    server_version=response.get("serverVersion", None),
                    open_subsonic=response.get("openSubsonic", None),
                )
            else:
                return ServerPingInfo(
                    is_ok=False,
                    error_code=response.get("error", {}).get("code", None),
                    error_message=response.get("error", {}).get("message", None),
                )
        except Exception as e:
            return ServerPingInfo(
                is_ok=False, error_message=f"Client error occurs: {e.__class__.__name__}: {e}"
            )

    async def get_music_folders(self) -> List[Dict]:
        """Get all music folders"""
        response = await self._make_request("getMusicFolders")
        return response.get("musicFolders", {}).get("musicFolder", [])

    async def get_artists(self, folder_id: Optional[str] = None) -> List[Dict]:
        """Get all artists in a music folder"""
        params = {"musicFolderId": folder_id} if folder_id else {}
        response = await self._make_request("getArtists", params)
        return response.get("artists", {}).get("index", [])

    async def get_artist(self, artist_id: str) -> Dict:
        """Get artist details"""
        response = await self._make_request("getArtist", {"id": artist_id})
        return response.get("artist", {})

    async def get_album(self, album_id: str) -> Dict:
        """Get album details"""
        response = await self._make_request("getAlbum", {"id": album_id})
        return response.get("album", {})

    async def get_song(self, song_id: str) -> Dict:
        """Get song details"""
        response = await self._make_request("getSong", {"id": song_id})
        return response.get("song", {})

    async def get_random_songs(
        self, size: int = 1, genre: Optional[str] = None, folder_id: Optional[str] = None
    ) -> List[Dict]:
        """Get random songs from the server"""
        params = {"size": size}
        if genre:
            params["genre"] = genre
        if folder_id:
            params["musicFolderId"] = folder_id

        response = await self._make_request("getRandomSongs", params)
        return response.get("randomSongs", {}).get("song", [])

    async def scrobble(self, song_id: str, submission: bool = True, time: Optional[int] = None) -> None:
        """
        Submit listening data to the server
        submission=True: The song was played fully
        submission=False: The song just started playing
        """
        params = {"id": song_id, "submission": "true" if submission else "false"}
        if time:
            params["time"] = time

        await self._make_request("scrobble", params)

    async def close(self):
        """Close the client and cleanup resources"""
        if self._session:
            await self._session.aclose()

    async def stream_noreturn(self, song_id: str) -> None:
        """Stream a song very slowly to simulate playback"""
        try:
            params = {
                "u": self.username,
                "t": self.token,
                "s": self.salt,
                "c": self.client,
                "v": self.version,
                "id": song_id,
            }
            url = urljoin(self.server, "rest/stream")

            session = await self._get_session()
            async with session.stream("GET", url, params=params) as resp:
                resp.raise_for_status()
                async for _ in resp.aiter_bytes(chunk_size=4096):
                    await asyncio.sleep(random.uniform(5, 10))
        except asyncio.CancelledError:
            pass



from ._base import BotCheckin

__ignore__ = True


class NiaogeCheckin(BotCheckin):
    name = "鸟哥轰炸"
    bot_username = "nb3344bot"
    bot_checkin_cmd = "/qd"
    additional_auth = ["prime"]
    bot_checked_keywords = "请勿重复签到"



from pyrogram.types import Message

from ._base import Monitor


class ViperMonitor(Monitor):
    name = "Viper"
    chat_keyword = r"register-[-\w]{36}"
    bot_username = "viper_emby_bot"
    chat_name = "Viper_Emby_Chat"
    notify_create_name = True
    allow_edit = False
    additional_auth = ["prime"]

    async def on_trigger(self, message: Message, key, reply):
        await self.client.send_message(self.bot_username, f"/invite {key}")
        await self.client.send_message(self.bot_username, self.unique_name)
        self.log.bind(msg=True).info(f'已向Bot发送邀请码: "{key}", 请查看.')



from logging import Formatter
import asyncio

from loguru import logger
from rich.logging import RichHandler

from . import var
from .utils import to_iterable

scheme_names = {
    "telegram": "Telegram",
    "telechecker": "每日签到",
    "telemonitor": "消息监控",
    "telemessager": "定时水群",
    "telelink": "账号服务",
    "telenotifier": "消息推送",
    "embywatcher": "Emby保活",
    "subsonic": "Subsonic保活",
    "datamanager": "下载器",
    "debugtool": "开发工具",
}


def formatter(record):
    """根据日志器的 scheme 属性配置输出格式."""
    extra = record["extra"]
    scheme = extra.get("scheme", None)

    def ifextra(keys, pattern="{}"):
        keys = to_iterable(keys)
        if all(k in extra for k in keys):
            return pattern.format(*[f"{{extra[{k}]}}" for k in keys])
        else:
            return ""

    if scheme in ("telegram", "telechecker", "telemonitor", "telemessager", "telelink"):
        username = ifextra("username", " ([cyan]{}[/])")
        name = ifextra("name", "([magenta]{}[/]) ")
        return f"[blue]{scheme_names[scheme]}[/]{username}: {name}{{message}}"
    elif scheme == "embywatcher":
        ident = ifextra(["server", "username"], " ([cyan]{}:{}[/])")
        return f"[blue]{scheme_names[scheme]}[/]{ident}: {{message}}"
    elif scheme == "subsonic":
        ident = ifextra(["server", "username"], " ([cyan]{}:{}[/])")
        return f"[blue]{scheme_names[scheme]}[/]{ident}: {{message}}"
    elif scheme == "datamanager":
        return f"[blue]{scheme_names[scheme]}[/]: {{message}}"
    elif scheme == "debugtool":
        return f"[blue]{scheme_names[scheme]}[/]: {{message}}"
    else:
        return "{message}"


def initialize(level="INFO", **kw):
    """初始化日志配置."""
    logger.remove()
    handler = RichHandler(
        console=var.console, markup=True, rich_tracebacks=True, tracebacks_suppress=[asyncio], **kw
    )
    handler.setFormatter(Formatter(None, "[%m/%d %H:%M]"))
    logger.add(handler, format=formatter, level=level, colorize=False)



try:
    import trio  # fix https://github.com/python-trio/trio/issues/3015
except ImportError:
    pass
from eventlet.patcher import monkey_patch

monkey_patch()

import binascii
import base64
import re
import atexit
import os
import pty
import select
import fcntl
import struct
from subprocess import Popen, PIPE
import termios
import threading
import time
import signal

import tomlkit
import typer
from loguru import logger
from flask import Flask, render_template, request, redirect, url_for, jsonify, abort
from flask_socketio import SocketIO
from flask_login import LoginManager, login_user, login_required, current_user

from . import __version__

cli = typer.Typer()
app = Flask(__name__, static_folder="templates/assets")
app.config["SECRET_KEY"] = os.urandom(24)
socketio = SocketIO(app, cors_allowed_origins="*")
login_manager = LoginManager()
login_manager.init_app(app)
login_manager.login_view = "login"

app.config["lock"] = threading.Lock()
app.config["args"] = []
app.config["fd"] = None
app.config["proc"] = None
app.config["hist"] = ""
app.config["faillog"] = []
app.config["config"] = os.environ.get("EK_CONFIG", "")

version = f"V{__version__}"


class DummyUser:
    def is_authenticated(self):
        return True

    def is_active(self):
        return True

    def is_anonymous(self):
        return False

    def get_id(self):
        return 0


@login_manager.user_loader
def load_user(_):
    return DummyUser()


def exit_handler():
    proc = app.config["proc"]
    if proc:
        kill_proc(proc)


@app.route("/")
def index():
    return redirect(url_for("console"))


def is_authenticated():
    webpass = app.config.get("webpass", None)
    if (not webpass) or current_user.is_authenticated:
        return True
    else:
        return False


@app.route("/console")
@login_required
def console():
    return render_template("console.html", version=version)


@app.route("/login", methods=["GET"])
def login():
    return render_template("login.html", version=version)


@app.route("/login", methods=["POST"])
def login_submit():
    password = request.form.get("password", "")
    webpass = os.environ.get("EK_WEBPASS", "")
    if not webpass:
        emsg = "后台没有设置控制台密码, 无法登录."
    elif sum(t > time.time() - 3600 for t in app.config["faillog"][-5:]) == 5:
        emsg = "一小时内有过多次失败登录, 请稍后再试."
    else:
        if password == webpass:
            login_user(DummyUser())
            return redirect(request.args.get("next") or url_for("index"))
        else:
            emsg = "密码错误, 请重试."
            app.config["faillog"].append(time.time())
    return render_template("login.html", emsg=emsg, version=version)


@app.route("/config", methods=["GET"])
@login_required
def config():
    return render_template("config.html", version=version)


@app.route("/config/current", methods=["GET"])
def config_current():
    if not is_authenticated():
        return "Not authenticated", 401
    data = app.config["config"]
    if not data:
        return "Config missing", 404
    try:
        data = base64.b64decode(re.sub(r"\s+", "", data).encode()).decode()
    except binascii.Error:
        logger.error("Config string malformed.")
        return "Config malformed", 400
    if isinstance(data, bytes):
        logger.error("Config string malformed.")
        return "Config malformed", 400
    return jsonify(data), 200


@app.route("/config/example", methods=["GET"])
def config_example():
    if not is_authenticated():
        return "Not authenticated", 401
    example, _ = Popen(["embykeeper", "--example-config"], stdout=PIPE, text=True).communicate()
    return jsonify(example), 200


@app.route("/config/save", methods=["POST"])
def config_save():
    if not is_authenticated():
        return "Not authenticated", 401
    data = request.get_json().get("config")
    clean_data = tomlkit.dumps(tomlkit.parse(data))
    encoded_data = base64.b64encode(clean_data.encode()).decode()
    app.config["config"] = encoded_data
    return jsonify(encoded_data), 200


@app.route("/healthz")
def healthz():
    return "200 OK"


@app.route("/heartbeat")
def heartbeat():
    webpass = os.environ.get("EK_WEBPASS", "")
    password = request.args.get("pass", None)
    if (not password) or (not webpass):
        return abort(403)
    if password == webpass:
        if app.config["proc"] is None:
            start_proc()
            return jsonify({"status": "restarted", "pid": app.config["proc"].pid}), 201
        else:
            return jsonify({"status": "running", "pid": app.config["proc"].pid}), 200
    else:
        return abort(403)


@app.errorhandler(404)
def page_not_found(e):
    return render_template("404.html", version=version), 404


@socketio.on("pty-input", namespace="/pty")
def pty_input(data):
    if not is_authenticated():
        return
    with app.config["lock"]:
        if app.config["fd"]:
            i = data["input"].encode()
            os.write(app.config["fd"], i)


def set_size(fd, row, col, xpix=0, ypix=0):
    logger.debug(f"Resizing pty to: {row} {col}.")
    size = struct.pack("HHHH", row, col, xpix, ypix)
    fcntl.ioctl(fd, termios.TIOCSWINSZ, size)


@socketio.on("resize", namespace="/pty")
def resize(data):
    logger.debug("Received resize socketio signal.")
    if not is_authenticated():
        return
    with app.config["lock"]:
        if app.config["fd"]:
            set_size(app.config["fd"], data["rows"], data["cols"])


@socketio.on("connect", namespace="/pty")
def handle_connect():
    logger.debug(f"Console connected from {request.sid}")


@socketio.on("disconnect", namespace="/pty")
def handle_disconnect():
    logger.debug(f"Console disconnected from {request.sid}")


@socketio.on_error_default
def default_error_handler(e):
    logger.error(f"SocketIO error occurred: {str(e)}")


def read_and_forward_pty_output():
    threading.current_thread().name = "pty_reader"
    max_read_bytes = 1024 * 20
    while True:
        if app.config["fd"]:
            try:
                (data, _, _) = select.select([app.config["fd"]], [], [], 1.0)
                if data:
                    with app.config["lock"]:
                        if app.config["fd"]:
                            output = os.read(app.config["fd"], max_read_bytes).decode(errors="ignore")
                            app.config["hist"] += output
                            socketio.emit("pty-output", {"output": output}, namespace="/pty")
                        else:
                            break
            except (select.error, OSError):
                break
        else:
            break
    logger.debug("PTY reader task ended")


def disconnect_on_proc_exit(proc: Popen):
    returncode = proc.wait()
    if proc == app.config["proc"]:
        logger.debug(f"Command exited with return code {returncode}.")
        output = f"\r\n\n程序已退出, 返回值 {returncode}. " "\r\n请您刷新页面以重新启动程序."
        app.config["hist"] += output
        socketio.emit("pty-output", {"output": output}, namespace="/pty")


def start_proc(instant=False):
    master_fd, slave_fd = pty.openpty()
    args = ["embykeeper", *app.config["args"]]
    if instant:
        args.append("--instant")
    p = Popen(
        args,
        stdin=slave_fd,
        stdout=slave_fd,
        stderr=slave_fd,
        env={**os.environ, "EK_CONFIG": app.config["config"]},
        preexec_fn=os.setsid,
    )
    socketio.start_background_task(target=disconnect_on_proc_exit, proc=p)
    atexit.register(exit_handler)
    app.config["fd"] = master_fd
    app.config["proc"] = p
    logger.debug(f"Embykeeper started at: {p.pid}.")
    socketio.start_background_task(target=read_and_forward_pty_output)


@socketio.on("embykeeper_start", namespace="/pty")
def start(data, auth=True):
    logger.debug(f"Received embykeeper_start socketio signal from {request.sid}.")
    if not is_authenticated():
        logger.debug("Authentication failed.")
        return
    with app.config["lock"]:
        if app.config["fd"] and app.config["proc"] and app.config["proc"].poll() is None:
            logger.debug("Existing process found, resizing and sending history.")
            set_size(app.config["fd"], data["rows"], data["cols"])
            socketio.sleep(0.1)
            socketio.emit("pty-output", {"output": app.config["hist"]}, namespace="/pty", to=request.sid)
            logger.debug(f"Sent pty-output to {request.sid}, length: {len(app.config['hist'])}.")
        else:
            logger.debug("Starting new process.")
            start_proc(instant=data.get("instant", False))
            set_size(app.config["fd"], data["rows"], data["cols"])


def kill_proc(proc: Popen):
    proc.send_signal(signal.SIGINT)
    for _ in range(10):
        poll = proc.poll()
        if poll is not None:
            break
    else:
        proc.kill()
    logger.debug(f"Embykeeper killed: {proc.pid}.")


@socketio.on("embykeeper_kill", namespace="/pty")
def kill():
    logger.debug("Received embykeeper_kill socketio signal.")
    if not is_authenticated():
        return
    with app.config["lock"]:
        proc = app.config["proc"]
        if proc is not None:
            app.config["fd"] = None
            app.config["proc"] = None
            app.config["hist"] = ""
    if proc is not None:
        socketio.start_background_task(target=kill_proc, proc=proc)


@cli.command(context_settings={"ignore_unknown_options": True, "allow_extra_args": True})
def run(
    ctx: typer.Context,
    port: int = typer.Option(1818, envvar="PORT", show_envvar=False),
    host: str = "0.0.0.0",
    debug: bool = False,
    wait: bool = False,
):
    app.config["args"] = ctx.args
    if not wait:
        start_proc(instant=True)
    logger.info(f"Embykeeper webserver started at {host}:{port}.")
    socketio.run(app, port=port, host=host, debug=debug)


if __name__ == "__main__":
    cli()



import re
from urllib.parse import parse_qs, urlparse

from pyrogram import Client
from pyrogram.types import Message
from pyrogram.raw.functions.messages import RequestAppWebView, GetBotApp
from pyrogram.raw.types import InputBotAppShortName, InputBotAppID, WebViewResultUrl
from pyrogram.raw.types.bot_app import BotApp
from pyrogram.raw.types.messages import BotApp as MessageBotApp
from aiohttp import ClientSession, TCPConnector
from aiohttp_socks import ProxyConnector, ProxyTimeoutError, ProxyError, ProxyType
from faker import Faker

from ..link import Link
from ._base import BotCheckin

__ignore__ = True


class TembyCheckin(BotCheckin):
    name = "Temby"
    bot_username = "HiEmbyBot"
    bot_checkin_cmd = "/hi"
    bot_success_pat = r".*(\d+)"
    max_retries = 1
    additional_auth = ["captcha"]
    bot_account_fail_keywords = ["需要邀请码"]

    async def message_handler(self, client: Client, message: Message):
        if message.text and message.text == "请在一分钟内点击下方按钮完成签到":
            if message.reply_markup:
                buttons = [b for l in message.reply_markup.inline_keyboard for b in l]
                if len(buttons) == 1:
                    button = buttons[0]
                    if button.text == "签到":
                        if button.url:
                            url = await self.get_app_url(button.url)
                            result = await self.solve_captcha(url)
                            if result:
                                await self.on_text(message, result)
                                return
                            else:
                                self.log.error("签到失败: 验证码解析失败, 正在重试.")
                                await self.retry()
                                return

        return await super().message_handler(client, message)

    async def get_app_url(self, url: str):
        match = re.search(r"t\.me/(\w+)/(\w+)\?startapp=(\w+)", url)
        if not match:
            return None
        bot_username, app_short_name, star_param = match.groups()
        bot_peer = await self.client.resolve_peer(bot_username)
        app_spec = InputBotAppShortName(bot_id=bot_peer, short_name=app_short_name)
        message_app: MessageBotApp = await self.client.invoke(GetBotApp(app=app_spec, hash=0))
        app: BotApp = message_app.app
        input_app = InputBotAppID(id=app.id, access_hash=app.access_hash)
        webview: WebViewResultUrl = await self.client.invoke(
            RequestAppWebView(peer=bot_peer, start_param=star_param, platform="ios", app=input_app)
        )
        return webview.url

    async def solve_captcha(self, url: str):
        token = await Link(self.client).captcha("temby")
        if not token:
            return None
        else:
            scheme = urlparse(url)
            params = parse_qs(scheme.query)
            messageid = params.get("tgWebAppStartParam", [None])[0]
            url_submit = scheme._replace(query="", fragment="").geturl()
            if self.proxy:
                connector = ProxyConnector(
                    proxy_type=ProxyType[self.proxy["scheme"].upper()],
                    host=self.proxy["hostname"],
                    port=self.proxy["port"],
                    username=self.proxy.get("username", None),
                    password=self.proxy.get("password", None),
                )
            else:
                connector = TCPConnector()
            useragent = Faker().safari()
            headers = {
                "Referer": url,
                "User-Agent": useragent,
            }
            params = {
                "messageid": messageid,
                "url": "",
                "cf-turnstile-response": token,
            }
            try:
                async with ClientSession(connector=connector) as session:
                    async with session.get(url_submit, headers=headers, params=params) as resp:
                        result = await resp.text()
                        match = re.search(r"<h1>(.*)</h1>", result)
                        if match:
                            return match.group(1)
                        else:
                            return None
            except (ProxyTimeoutError, ProxyError, OSError):
                return None



from ._base import BotCheckin

__ignore__ = True


class BaiduSGKCheckin(BotCheckin):
    name = "度娘社工库"
    bot_username = "baidusell_bot"
    bot_checkin_cmd = "/sign"
    additional_auth = ["prime"]
    bot_checked_keywords = ["今日已签到"]



from __future__ import annotations

import asyncio
from datetime import datetime, time
from pathlib import Path
import random
from typing import TYPE_CHECKING, Iterable, Tuple, Union
import json
from urllib.parse import urlparse

import httpx
from loguru import logger

from .api import Subsonic

from ..utils import next_random_datetime, show_exception

if TYPE_CHECKING:
    from loguru import Logger

logger = logger.bind(scheme="subsonic")


async def listen(client: Subsonic, loggeruser: Logger, time: Union[float, Iterable[float]]):
    """模拟连续播放音频直到达到指定总时长."""

    if isinstance(time, Iterable):
        total_time = random.uniform(*time)
    else:
        total_time = float(time)

    played_time = 0
    max_retries = 3
    current_retries = 0

    while played_time < total_time:
        try:
            songs = await client.get_random_songs()
            if not songs:
                loggeruser.warning("未能获取到任何歌曲.")
                return False
            song = random.choice(songs)
            song_id = song.get("id", None)
            if not song_id:
                loggeruser.warning("获取到歌曲信息不完整, 正在重试.")
                continue

            song_title = song.get("title", "未知歌曲")
            song_duration = float(song.get("duration", 60))
            remaining_time = total_time - played_time

            play_duration = min(remaining_time, song_duration) if song_duration > 0 else remaining_time

            loggeruser.info(f'开始播放 "{song_title}", 剩余时间 {remaining_time:.0f} 秒.')
            while current_retries < max_retries:
                try:
                    await client.scrobble(song_id, submission=False)
                    await asyncio.wait_for(client.stream_noreturn(song_id), timeout=play_duration)
                    played_time += play_duration
                    await client.scrobble(song_id, submission=True)
                    loggeruser.info(f'完成播放 "{song_title}", 已播放 {played_time:.0f} 秒.')
                    current_retries = 0
                    break
                except asyncio.TimeoutError:
                    # 正常超时，说明歌曲播放完成
                    played_time += play_duration
                    await client.scrobble(song_id, submission=True)
                    loggeruser.info(f'完成播放 "{song_title}", 已播放 {played_time:.0f} 秒.')
                    break
                except Exception as e:
                    current_retries += 1
                    if current_retries >= max_retries:
                        loggeruser.error(f"播放出错且达到最大重试次数, 停止播放.")
                        show_exception(e, regular=False)
                        return False
                    loggeruser.warning(f"播放出错 (重试 {current_retries}/{max_retries}), 正在重试.")
                    show_exception(e, regular=False)
                    await asyncio.sleep(1)
                    continue
        except httpx.HTTPError as e:
            current_retries += 1
            if current_retries >= max_retries:
                loggeruser.error(f"播放出错且达到最大重试次数, 停止播放.")
                show_exception(e, regular=False)
                return False
            loggeruser.warning(f"访问出错 (重试 {current_retries}/{max_retries}), 正在重试: {e}.")
            show_exception(e, regular=True)
            await asyncio.sleep(1)
            continue
    return True


async def login(config):
    """登录账号."""
    for a in config.get("subsonic", ()):
        server_url = a["url"]
        domain = urlparse(server_url).netloc

        proxy = None
        client = Subsonic(
            server=server_url,
            username=a["username"],
            password=a["password"],
            proxy=proxy or config.get("proxy", None),
            ua=a.get("ua", None),
            client=a.get("client", None),
            version=a.get("version", None),
        )
        info = await client.ping()
        if info.is_ok:
            loggeruser = logger.bind(server=domain, username=a["username"])
            loggeruser.info(
                f'成功连接至服务器 "{domain}" ({(info.type or "unknown").capitalize()} {info.version or "X.X"}).'
            )
            yield (
                client,
                loggeruser,
                a.get("time", [120, 240]),
            )
        else:
            logger.bind(log=True).error(f'服务器 "{domain}" 登陆错误, 请重新检查配置: {info.error_message}')
            continue


async def listener(config: dict, instant: bool = False):
    """入口函数 - 收听一个音频."""

    async def wrapper(
        sem: asyncio.Semaphore,
        client: Subsonic,
        loggeruser: Logger,
        time: float,
    ):
        async with sem:
            try:
                if not instant:
                    wait = random.uniform(180, 360)
                    loggeruser.info(f"播放音频前随机等待 {wait:.0f} 秒.")
                    await asyncio.sleep(wait)
                if isinstance(time, Iterable):
                    tm = max(time) * 4
                else:
                    tm = time * 4
                return await asyncio.wait_for(listen(client, loggeruser, time), max(tm, 600))
            except asyncio.TimeoutError:
                loggeruser.warning(f"一定时间内未完成播放, 保活失败.")
                return False

    logger.info("开始执行 Subsonic 保活.")
    tasks = []
    concurrent = int(config.get("listen_concurrent", 3))
    if not concurrent:
        concurrent = 100000
    sem = asyncio.Semaphore(concurrent)
    async for client, loggeruser, time in login(config):
        tasks.append(wrapper(sem, client, loggeruser, time))
    if not tasks:
        logger.info("没有指定相关的 Subsonic 服务器, 跳过保活.")
    results = await asyncio.gather(*tasks)
    fails = len(tasks) - sum(results)
    if fails:
        logger.error(f"保活失败 ({fails}/{len(tasks)}).")


async def listener_schedule(
    config: dict,
    start_time=time(11, 0),
    end_time=time(23, 0),
    days: Union[int, Tuple[int, int]] = 7,
    instant: bool = False,
):
    """计划任务 - 收听一个音频."""

    timestamp_file = Path(config["basedir"]) / "listener_schedule_next_timestamp"
    current_config = {
        "start_time": start_time.strftime("%H:%M"),
        "end_time": end_time.strftime("%H:%M"),
        "days": days if isinstance(days, int) else list(days),
    }

    while True:
        next_dt = None
        config_changed = False

        if timestamp_file.exists():
            try:
                stored_data = json.loads(timestamp_file.read_text())
                if not isinstance(stored_data, dict):
                    raise ValueError("invalid cache")
                stored_timestamp = stored_data["timestamp"]
                stored_config = stored_data["config"]

                if stored_config != current_config:
                    logger.info("计划任务配置已更改，将重新计算下次执行时间.")
                    config_changed = True
                else:
                    next_dt = datetime.fromtimestamp(stored_timestamp)
                    if next_dt > datetime.now():
                        logger.info(f"从缓存中读取到下次保活时间: {next_dt.strftime('%m-%d %H:%M %p')}.")
            except (ValueError, OSError, json.JSONDecodeError) as e:
                logger.debug(f"读取存储的时间戳失败: {e}")
                config_changed = True

        if not next_dt or next_dt <= datetime.now() or config_changed:
            if isinstance(days, int):
                rand_days = days
            else:
                rand_days = random.randint(*days)
            next_dt = next_random_datetime(start_time, end_time, interval_days=rand_days)
            logger.info(f"下一次保活将在 {next_dt.strftime('%m-%d %H:%M %p')} 进行.")

            try:
                save_data = {"timestamp": next_dt.timestamp(), "config": current_config}
                timestamp_file.write_text(json.dumps(save_data))
            except OSError as e:
                logger.debug(f"存储时间戳失败: {e}")

        await asyncio.sleep((next_dt - datetime.now()).total_seconds())
        try:
            timestamp_file.unlink(missing_ok=True)
        except OSError as e:
            logger.debug(f"删除时间戳文件失败: {e}")
        await listener(config, instant=instant)



import asyncio
import re

from pyrogram import Client
from pyrogram.types import Message
from ..link import Link
from ._base import BotCheckin

__ignore__ = True


class CharonCheckin(BotCheckin):
    name = "卡戎"
    bot_username = "charontv_bot"
    bot_success_pat = r".*(\d+)"
    bot_text_ignore = ["已结束当前对话"]
    additional_auth = ["captcha"]

    async def send_checkin(self, retry=False):
        if retry:
            await asyncio.sleep(self.bot_send_interval)
        while True:
            await self.send("/checkin")
            if await self.wait_until("已结束当前对话", 3):
                await asyncio.sleep(self.bot_send_interval)
                continue
            else:
                break

    async def message_handler(self, client: Client, message: Message):
        if message.text:
            match = re.search(r"请打开并复制网页的内容, 粘贴回复:\s*(.*)", message.text)
            if match:
                return await self.handle_url(match.group(1))
        return await super().message_handler(client, message)

    async def handle_url(self, url: str):
        self.log.debug(f"即将解析网页中的验证码: {url}.")
        for i in range(3):
            _, result = await Link(self.client).captcha_url("charon", url)
            if result:
                await self.client.send_message(self.bot_username, result)
                break
            else:
                self.log.warning(f"正在重试解析验证码 ({i+1} / 3).")



from ._base import BotCheckin

__ignore__ = True


class BostCheckin(BotCheckin):
    name = "Bost 社工库"
    bot_username = "BOST_SGK_BOT"
    bot_checkin_cmd = "/qd"
    additional_auth = ["prime"]
    bot_use_captcha = False



from ._base import BotCheckin

__ignore__ = True


class MoshenSGKCheckin(BotCheckin):
    name = "魔神社工库"
    bot_username = "moshensgk_bot"
    bot_checkin_cmd = "/qd"
    additional_auth = ["prime"]
    max_retries = 6



from ._base import BotCheckin

__ignore__ = True


class BearSGKCheckin(BotCheckin):
    name = "小熊社工库"
    bot_username = "BearSGK_bot"
    bot_checkin_cmd = "/sign"
    additional_auth = ["prime"]
    bot_checked_keywords = "请勿重复签到"



from ._base import BotCheckin

__ignore__ = True


class QingfengSGKCheckin(BotCheckin):
    name = "清风社工库"
    bot_username = "Weifeng007_bot"
    bot_checkin_cmd = "/qd"
    additional_auth = ["prime"]
    bot_checked_keywords = "您今天已经签到过了"



import asyncio

from pyrogram import Client
from pyrogram.enums import ChatType
from pyrogram.types import Message
from pyrogram.errors import RPCError
from cachetools import TTLCache

from ._base import Monitor

__ignore__ = True


class FollowMonitor(Monitor):
    name = "全部群组从众"
    lock = asyncio.Lock()
    cache = TTLCache(maxsize=2048, ttl=300)
    chat_follow_user = 5

    async def start(self):
        async with self.listener():
            self.log.info(f"开始监视: {self.name}.")
            await self.failed.wait()
            self.log.error(f"发生错误, 不再监视: {self.name}.")
            return False

    async def message_handler(self, client: Client, message: Message):
        if not message.text:
            return
        if message.chat.type not in (ChatType.GROUP, ChatType.SUPERGROUP):
            return
        if len(message.text) > 50:
            return
        if message.text.startswith("/"):
            return
        if not message.from_user:
            return
        if message.from_user.is_bot:
            return
        ident = (message.chat.id, message.text)
        async with self.lock:
            if ident in self.cache:
                self.cache[ident] += 1
                if self.cache[ident] == self.chat_follow_user:
                    try:
                        chat_id, text = ident
                        await self.client.send_message(chat_id, text)
                    except RPCError as e:
                        self.log.warning(f"发送从众信息到群组 {message.chat.title} 失败: {e}.")
                    else:
                        self.log.info(f"已发送从众信息到群组 {message.chat.title}.")
            else:
                self.cache[ident] = 1



import emoji
from pyrogram.types import Message
from pyrogram.errors import RPCError

from ..link import Link
from ._base import AnswerBotCheckin


class TerminusCheckin(AnswerBotCheckin):
    name = "终点站"
    bot_username = "EmbyPublicBot"
    bot_checkin_cmd = ["/cancel", "/checkin"]
    bot_text_ignore = ["会话已取消", "没有活跃的会话"]
    bot_checked_keywords = ["今天已签到"]
    additional_auth = ["visual"]
    max_retries = 1
    bot_use_history = 3

    async def on_photo(self, message: Message):
        """分析分析传入的验证码图片并返回验证码."""
        if message.reply_markup:
            clean = lambda o: emoji.replace_emoji(o, "").replace(" ", "")
            keys = [k for r in message.reply_markup.inline_keyboard for k in r]
            options = [k.text for k in keys]
            options_cleaned = [clean(o) for o in options]
            if len(options) < 2:
                return
            for i in range(3):
                result, by = await Link(self.client).visual(message.photo.file_id, options_cleaned)
                if result:
                    self.log.debug(f"已通过远端 ({by}) 解析答案: {result}.")
                    break
                else:
                    self.log.warning(f"远端解析失败, 正在重试解析 ({i + 1}/3).")
            else:
                self.log.warning(f"签到失败: 验证码识别错误.")
                return await self.fail()
            result = options[options_cleaned.index(result)]
            try:
                await message.click(result)
            except RPCError:
                self.log.warning("按钮点击失败.")



import asyncio
from pyrogram.types import Message
from pyrogram import filters

from embykeeper.utils import async_partial

from ._base import Monitor

__ignore__ = True


class ShufuMonitor(Monitor):
    name = "叔服"
    chat_name = -1001890341413
    chat_keyword = r"SHUFU-\d+-Register_[\w]+"
    bot_username = "dashu660_bot"
    notify_create_name = True
    additional_auth = ["prime"]

    async def on_trigger(self, message: Message, key, reply):
        wr = async_partial(self.client.wait_reply, self.bot_username)
        for _ in range(3):
            try:
                msg = await wr("/start")
                if "请确认好重试" in (msg.text or msg.caption):
                    continue
                elif "欢迎进入用户面板" in (msg.text or msg.caption) and msg.reply_markup:
                    keys = [k.text for r in msg.reply_markup.inline_keyboard for k in r]
                    for k in keys:
                        if "使用注册码" in k:
                            async with self.client.catch_reply(
                                self.bot_username, filter=filters.regex(".*对我发送.*")
                            ) as f:
                                try:
                                    await msg.click(k)
                                except TimeoutError:
                                    pass
                                try:
                                    await asyncio.wait_for(f, 10)
                                except asyncio.TimeoutError:
                                    continue
                                else:
                                    break
                    else:
                        continue
                    msg = await wr(key)
                    if "注册码已被使用" in (msg.text or msg.caption):
                        self.log.info(f'已向 Bot @{self.bot_username} 发送了邀请码: "{key}", 但是已被抢注了.')
                    else:
                        self.log.bind(msg=True).info(
                            f'已向 Bot @{self.bot_username} 发送了邀请码: "{key}", 请查看.'
                        )
                    break
                else:
                    continue
            except asyncio.TimeoutError:
                pass
        else:
            self.log.bind(msg=True).warning(f"已监控到{self.name}的邀请码, 但自动使用失败, 请自行查看.")



from pathlib import Path
from datetime import datetime, timedelta
import re
import sys
from typing import List

import typer
import asyncio
from dateutil import parser

from . import var, __author__, __name__, __url__, __version__
from .utils import Flagged, FlagValueCommand, AsyncTyper, AsyncTaskPool, show_exception
from .settings import prepare_config

app = AsyncTyper(
    pretty_exceptions_enable=False,
    rich_markup_mode="rich",
    add_completion=False,
    context_settings={"help_option_names": ["-h", "--help"]},
)


def version(flag):
    if flag:
        print(__version__)
        raise typer.Exit()


def print_example_config(flag):
    if flag:
        import io

        from .settings import write_faked_config

        file = io.StringIO()
        write_faked_config(file, quiet=True)
        file.seek(0)
        print(file.read())
        raise typer.Exit()


@app.async_command(
    cls=FlagValueCommand,
    help=f"欢迎使用 [orange3]{__name__.capitalize()}[/] {__version__} :cinema: 无参数默认开启全部功能.",
)
async def main(
    config: Path = typer.Argument(
        None,
        dir_okay=False,
        allow_dash=True,
        envvar=f"EK_CONFIG_FILE",
        rich_help_panel="参数",
        help="配置文件 (置空以生成)",
    ),
    checkin: str = typer.Option(
        Flagged("", "-"),
        "--checkin",
        "-c",
        rich_help_panel="模块开关",
        show_default="不指定值时默认为8:00AM-10:00AM之间随机时间",
        help="启用每日指定时间签到",
    ),
    emby: str = typer.Option(
        Flagged("", "-"),
        "--emby",
        "-e",
        rich_help_panel="模块开关",
        help="启用每隔天数Emby自动保活",
        show_default="不指定值时默认为每3-12天",
    ),
    subsonic: str = typer.Option(
        Flagged("", "-"),
        "--subsonic",
        rich_help_panel="模块开关",
        help="启用每隔天数Subsonic自动保活",
        show_default="不指定值时默认为3-12天",
    ),
    monitor: bool = typer.Option(False, "--monitor", "-m", rich_help_panel="模块开关", help="启用群聊监视"),
    send: bool = typer.Option(False, "--send", "-s", rich_help_panel="模块开关", help="启用自动水群"),
    version: bool = typer.Option(
        None,
        "--version",
        "-v",
        rich_help_panel="调试参数",
        callback=version,
        is_eager=True,
        help=f"打印 {__name__.capitalize()} 版本",
    ),
    example_config: bool = typer.Option(
        None,
        "--example-config",
        "-E",
        hidden=True,
        callback=print_example_config,
        is_eager=True,
        help=f"输出范例配置文件",
    ),
    instant: bool = typer.Option(
        False,
        "--instant/--no-instant",
        "-i/-I",
        envvar="EK_INSTANT",
        show_envvar=False,
        rich_help_panel="调试参数",
        help="启动时立刻执行一次任务",
    ),
    once: bool = typer.Option(
        False, "--once/--cron", "-o/-O", rich_help_panel="调试参数", help="不等待计划执行"
    ),
    verbosity: int = typer.Option(
        False,
        "--debug",
        "-d",
        count=True,
        envvar="EK_DEBUG",
        show_envvar=False,
        rich_help_panel="调试参数",
        help="开启调试模式",
    ),
    debug_cron: bool = typer.Option(
        False,
        envvar="EK_DEBUG_CRON",
        show_envvar=False,
        help="开启任务调试模式, 在三秒后立刻开始执行计划任务",
    ),
    debug_notify: bool = typer.Option(
        False,
        show_envvar=False,
        help="开启日志调试模式, 发送一条日志记录和即时日志记录后退出",
    ),
    simple_log: bool = typer.Option(
        False, "--simple-log", "-L", rich_help_panel="调试参数", help="简化日志输出格式"
    ),
    disable_color: bool = typer.Option(
        False, "--disable-color", "-C", rich_help_panel="调试参数", help="禁用日志颜色"
    ),
    follow: bool = typer.Option(False, "--follow", "-F", rich_help_panel="调试工具", help="仅启动消息调试"),
    analyze: bool = typer.Option(
        False, "--analyze", "-A", rich_help_panel="调试工具", help="仅启动历史信息分析"
    ),
    dump: List[str] = typer.Option([], "--dump", "-D", rich_help_panel="调试工具", help="仅启动更新日志"),
    save: bool = typer.Option(
        False, "--save", "-S", rich_help_panel="调试参数", help="记录执行过程中的原始更新日志"
    ),
    public: bool = typer.Option(
        False,
        "--public",
        "-P",
        hidden=True,
        rich_help_panel="调试参数",
        help="启用公共仓库部署模式",
    ),
    windows: bool = typer.Option(
        False,
        "--windows",
        "-W",
        hidden=True,
        rich_help_panel="调试参数",
        help="启用 Windows 安装部署模式",
    ),
    basedir: Path = typer.Option(
        None, "--basedir", "-B", rich_help_panel="调试参数", help="设定账号文件和模型文件的位置"
    ),
):
    from .log import logger, initialize

    var.debug = verbosity
    if verbosity >= 3:
        level = 0
    elif verbosity >= 1:
        level = "DEBUG"
    else:
        level = "INFO"

    initialize(level=level, show_path=verbosity and (not simple_log), show_time=not simple_log)
    if disable_color:
        var.console.no_color = True

    msg = " 您可以通过 Ctrl+C 以结束运行." if not public else ""
    logger.info(f"欢迎使用 [orange3]{__name__.capitalize()}[/]! 正在启动, 请稍等.{msg}")
    logger.info(f"当前版本 ({__version__}) 项目页: {__url__}")
    logger.debug(f'命令行参数: "{" ".join(sys.argv[1:])}".')

    if verbosity:
        logger.warning(f"您当前处于调试模式: 日志等级 {verbosity}.")
        app.pretty_exceptions_enable = True

    config: dict = await prepare_config(config, basedir=basedir, public=public, windows=windows)

    if verbosity >= 2:
        config["nofail"] = False
    if not config.get("nofail", True):
        logger.warning(f"您当前处于调试模式: 错误将会导致程序停止运行.")
    if debug_cron:
        logger.warning("您当前处于计划任务调试模式, 将在 10 秒后运行计划任务.")

    default_time = config.get("time", "<8:00AM,10:00AM>")
    default_interval = config.get("interval", "<3,12>")
    logger.debug(f"采用默认签到时间范围 {default_time}, 默认保活间隔天数 {default_interval}.")

    if checkin == "-":
        checkin = default_time

    if emby == "-":
        emby = default_interval

    if subsonic == "-":
        subsonic = default_interval

    if not checkin and not monitor and not emby and not send and not subsonic:
        checkin = default_time
        emby = default_interval
        subsonic = default_interval
        monitor = True
        send = True

    if follow:
        from .telechecker.debug import follower

        return await follower(config)

    if analyze:
        from .telechecker.debug import analyzer

        indent = " " * 23
        chats = typer.prompt(indent + "请输入群组用户名 (以空格分隔)").split()
        keywords = typer.prompt(indent + "请输入关键词 (以空格分隔)", default="", show_default=False)
        keywords = keywords.split() if keywords else []
        timerange = typer.prompt(indent + '请输入时间范围 (以"-"分割)', default="", show_default=False)
        timerange = timerange.split("-") if timerange else []
        limit = typer.prompt(indent + "请输入各群组最大获取数量", default=10000, type=int)
        outputs = typer.prompt(indent + "请输入最大输出数量", default=1000, type=int)
        return await analyzer(config, chats, keywords, timerange, limit, outputs)

    if dump:
        from .telechecker.debug import dumper

        return await dumper(config, dump)

    if debug_notify:
        from .telechecker.notify import start_notifier

        if await start_notifier(config):
            logger.info("以下是发送的日志:")
            logger.bind(msg=True, scheme="debugtool").info(
                "这是一条用于测试的即时消息, 使用 debug_notify 触发 😉."
            )
            logger.bind(log=True, scheme="debugtool").info(
                "这是一条用于测试的日志消息, 使用 debug_notify 触发 😉."
            )
            logger.info("已尝试发送, 请至 @embykeeper_bot 查看.")
            await asyncio.sleep(10)
        else:
            logger.error("您当前没有配置有效的日志通知 (未启用日志通知或未配置账号), 请检查配置文件.")
        return

    if emby and not isinstance(emby, int):
        try:
            emby = abs(int(emby))
        except ValueError:
            interval_range_match = re.match(r"<(\d+),(\d+)>", emby)
            if interval_range_match:
                emby = [int(interval_range_match.group(1)), int(interval_range_match.group(2))]
            else:
                logger.error(f"无法解析 Emby 保活间隔天数: {default_interval}, 保活将不会运行.")
                emby = False

    if subsonic and not isinstance(subsonic, int):
        try:
            subsonic = abs(int(subsonic))
        except ValueError:
            interval_range_match = re.match(r"<(\d+),(\d+)>", subsonic)
            if interval_range_match:
                subsonic = [int(interval_range_match.group(1)), int(interval_range_match.group(2))]
            else:
                logger.error(f"无法解析 Subsonic 保活间隔天数: {default_interval}, 保活将不会运行.")
                subsonic = False

    from .telechecker.notify import start_notifier

    if emby:
        from .embywatcher.main import (
            watcher,
            watcher_schedule,
            watcher_continuous_schedule,
        )
    if subsonic:
        from .subsonic.main import (
            listener,
            listener_schedule,
        )
    if checkin or monitor or send:
        from .telechecker.main import (
            checkiner,
            checkiner_schedule,
            messager,
            monitorer,
        )

    pool = AsyncTaskPool()

    if save:
        from .telechecker.debug import saver

        asyncio.create_task(saver(config))

    if instant and not debug_cron:
        if emby:
            pool.add(watcher(config, instant=True))
        if checkin:
            pool.add(checkiner(config, instant=True))
        if subsonic:
            pool.add(listener(config, instant=True))
        await pool.wait()
        logger.debug("启动时立刻执行签到和保活: 已完成.")

    if not once:
        streams = await start_notifier(config)
        try:
            if emby:
                try:
                    if debug_cron:
                        start_time = end_time = (datetime.now() + timedelta(seconds=10)).time()
                    else:
                        watchtime = config.get("watchtime", "<11:00AM,11:00PM>")
                        watchtime_match = re.match(r"<\s*(.*),\s*(.*)\s*>", watchtime)
                        if watchtime_match:
                            start_time, end_time = [
                                parser.parse(watchtime_match.group(i)).time() for i in (1, 2)
                            ]
                        else:
                            start_time = end_time = parser.parse(watchtime).time()
                except parser.ParserError:
                    logger.error(
                        "您设定的 watchtime 不正确, 请检查格式. (例如 11:00, <11:00,14:00> / <11:00AM,2:00PM>). 模拟观看保活将不会运行."
                    )
                else:
                    pool.add(
                        watcher_schedule(
                            config,
                            days=0 if debug_cron else emby,
                            start_time=start_time,
                            end_time=end_time,
                        )
                    )
                    for a in config.get("emby", ()):
                        if a.get("continuous", False):
                            pool.add(
                                watcher_continuous_schedule(
                                    config,
                                    days=0 if debug_cron else 1,
                                    start_time=start_time,
                                    end_time=end_time,
                                )
                            )
                            break
            if subsonic:
                try:
                    if debug_cron:
                        start_time = end_time = (datetime.now() + timedelta(seconds=10)).time()
                    else:
                        listentime = config.get("listentime", "<11:00AM,11:00PM>")
                        listentime_match = re.match(r"<\s*(.*),\s*(.*)\s*>", listentime)
                        if listentime:
                            start_time, end_time = [
                                parser.parse(listentime_match.group(i)).time() for i in (1, 2)
                            ]
                        else:
                            start_time = end_time = parser.parse(listentime).time()
                except parser.ParserError:
                    logger.error(
                        "您设定的 listentime 不正确, 请检查格式. (例如 11:00, <11:00,14:00> / <11:00AM,2:00PM>). 模拟观看保活将不会运行."
                    )
                else:
                    pool.add(
                        listener_schedule(
                            config,
                            days=0 if debug_cron else subsonic,
                            start_time=start_time,
                            end_time=end_time,
                        )
                    )
            if checkin:
                try:
                    if debug_cron:
                        start_time = end_time = (datetime.now() + timedelta(seconds=10)).time()
                    else:
                        checkin_range_match = re.match(r"<\s*(.*),\s*(.*)\s*>", checkin)
                        if checkin_range_match:
                            start_time, end_time = [
                                parser.parse(checkin_range_match.group(i)).time() for i in (1, 2)
                            ]
                        else:
                            start_time = end_time = parser.parse(checkin).time()
                except parser.ParserError:
                    logger.error(
                        "您设定的 time 不正确, 请检查格式. (例如 11:00, <11:00,14:00> / <11:00AM,2:00PM>). 自动签到将不会运行."
                    )
                else:
                    pool.add(
                        checkiner_schedule(
                            config,
                            instant=False,
                            start_time=start_time,
                            end_time=end_time,
                            days=0 if debug_cron else 1,
                        )
                    )
            if monitor:
                pool.add(monitorer(config))
            if send:
                pool.add(messager(config))

            async for t in pool.as_completed():
                msg = f"任务 {t.get_name()} "
                try:
                    e = t.exception()
                    if e:
                        msg += f"发生错误并退出: {e}"
                    else:
                        msg += f"成功结束."
                except asyncio.CancelledError:
                    msg += f"被取消."
                logger.debug(msg)
                try:
                    await t
                except Exception as e:
                    logger.error("出现错误, 模块可能停止运行.")
                    show_exception(e, regular=False)
                    if not config.get("nofail", True):
                        raise
        except asyncio.CancelledError:
            if streams:
                await asyncio.gather(*[stream.join() for stream in streams])


if __name__ == "__main__":
    app()



from ._base import BotCheckin

__ignore__ = True


class Carll1SGKCheckin(BotCheckin):
    name = "Carll 社工库 1"
    bot_username = "Carllnet_bot"
    bot_checkin_cmd = "/qd"
    additional_auth = ["prime"]



from pyrogram.types import Message

from ._base import Monitor


class BGKMonitor(Monitor):
    name = "不给看"
    chat_name = "Ephemeralemby"
    chat_keyword = r"(?:^|\s)([a-zA-Z0-9]{32})(?!\S)"
    bot_username = "UnknownEmbyBot"
    notify_create_name = True
    additional_auth = ["prime"]

    async def on_trigger(self, message: Message, key, reply):
        await self.client.send_message(self.bot_username, "/invite")
        await self.client.send_message(self.bot_username, key)
        await self.client.send_message(self.bot_username, self.unique_name)
        self.log.bind(msg=True).info(f'已向 Bot @{self.bot_username} 发送了邀请码: "{key}", 请查看.')



import logging

from loguru import logger

from ..log import formatter
from .log import TelegramStream

logger = logger.bind(scheme="telegram")


async def start_notifier(config: dict):
    """消息通知初始化函数."""

    def _filter_log(record):
        notify = record.get("extra", {}).get("log", None)
        if notify or record["level"].no == logging.ERROR:
            return True
        else:
            return False

    def _filter_msg(record):
        notify = record.get("extra", {}).get("msg", None)
        if notify:
            return True
        else:
            return False

    def _formatter(record):
        return "{level}#" + formatter(record)

    accounts = config.get("telegram", [])
    notifier = config.get("notifier", None)
    if notifier:
        try:
            if notifier == True:
                notifier = accounts[0]
            elif isinstance(notifier, int):
                notifier = accounts[notifier + 1]
            elif isinstance(notifier, str):
                for a in accounts:
                    if a["phone"] == notifier:
                        notifier = a
                        break
            else:
                notifier = None
        except IndexError:
            notifier = None
    if notifier:
        logger.info(f'计划任务的关键消息将通过 Embykeeper Bot 发送至 "{notifier["phone"]}" 账号.')
        stream_log = TelegramStream(
            account=notifier,
            proxy=config.get("proxy", None),
            basedir=config.get("basedir", None),
            instant=config.get("notify_immediately", False),
        )
        logger.add(
            stream_log,
            format=_formatter,
            filter=_filter_log,
        )
        stream_msg = TelegramStream(
            account=notifier,
            proxy=config.get("proxy", None),
            basedir=config.get("basedir", None),
            instant=True,
        )
        logger.add(
            stream_msg,
            format=_formatter,
            filter=_filter_msg,
        )
        return stream_log, stream_msg
    else:
        return None



import asyncio

from ._base import BotCheckin

__ignore__ = True


class CharonCheckin(BotCheckin):
    name = "卡戎"
    ocr = "words6@v1"
    bot_username = "charontv_bot"
    bot_checkin_cmd = ["/checkin", "/cancel"]
    bot_captcha_len = 6
    bot_success_pat = r".*(\d+)"
    bot_text_ignore = ["已结束当前对话"]

    async def send_checkin(self, retry=False):
        if retry:
            await asyncio.sleep(self.bot_send_interval)
        while True:
            await self.send("/checkin")
            if await self.wait_until("已结束当前对话", 3):
                continue
            else:
                break



from ._base import BotCheckin

__ignore__ = True


class InfSGKCheckin(BotCheckin):
    name = "情报局社工库"
    bot_username = "qbjSGKzhuquebot"
    bot_checkin_cmd = "/sign"
    bot_success_keywords = ["签到成功"]
    bot_checked_keywords = ["您已经签到过了"]
    checked_retries = 6



from datetime import datetime, timedelta
import random
import re

import asyncio
from typing import List
from cachetools import TTLCache
from pyrogram.types import Message, User, Chat
from pyrogram.enums import ChatMemberStatus, MessageServiceType, MessagesFilter
from pyrogram.errors import BadRequest

from ..lock import pornemby_alert, pornemby_messager_mids
from ._base import Monitor


class PornembyAlertMonitor(Monitor):
    name = "Pornemby 风险急停监控"
    chat_name = "Pornemby"
    additional_auth = ["pornemby_pack"]
    allow_edit = True
    debug_no_log = True

    user_alert_keywords = ["脚本", "真人", "admin", "全是", "举报", "每次", "机器人", "report"]
    admin_alert_keywords = ["不要", "封", "ban", "warn", "踢", "抓"]
    alert_reply_keywords = ["真人", "脚本", "每次", "在吗", "机器", "封", "warn", "ban", "回", "说"]
    alert_reply_except_keywords = ["不要回复", "别回复", "勿回复"]
    reply_words = ["?" * (i + 1) for i in range(3)] + ["嗯?", "欸?", "🤔"]
    reply_interval = 7200

    async def init(self):
        self.lock = asyncio.Lock()
        self.last_reply = None
        self.alert_remaining = 0.0
        self.member_status_cache = TTLCache(maxsize=128, ttl=86400)
        self.member_status_cache_lock = asyncio.Lock()
        self.monitor_task = asyncio.create_task(self.monitor())
        self.pin_checked = False
        return True

    async def check_admin(self, chat: Chat, user: User):
        if not user:
            return True
        async with self.member_status_cache_lock:
            if not user.id in self.member_status_cache:
                try:
                    member = await self.client.get_chat_member(chat.id, user.id)
                    self.member_status_cache[user.id] = member.status
                except BadRequest:
                    return False
        if self.member_status_cache[user.id] in (ChatMemberStatus.ADMINISTRATOR, ChatMemberStatus.OWNER):
            if user.is_bot:
                return False
            else:
                return True

    def check_keyword(self, message: Message, keywords: List[str]):
        content = message.text or message.caption
        if content:
            for k in keywords:
                match = re.search(k, content)
                if match:
                    return match.group(0)

    async def monitor(self):
        while True:
            await self.lock.acquire()
            while self.alert_remaining > 0:
                pornemby_alert[self.client.me.id] = True
                t = datetime.now()
                self.lock.release()
                await asyncio.sleep(1)
                await self.lock.acquire()
                self.alert_remaining -= (datetime.now() - t).total_seconds()
            else:
                pornemby_alert[self.client.me.id] = False
            self.lock.release()
            await asyncio.sleep(1)

    async def set_alert(self, time: float = None, reason: str = None):
        if time:
            async with self.lock:
                if self.alert_remaining > time:
                    return
                else:
                    msg = f"Pornemby 风险急停被触发, 停止操作 {time} 秒"
                    if reason:
                        msg += f" (原因: {reason})"
                    msg += "."
                    self.log.warning(msg)
                    self.alert_remaining = time
        else:
            msg = "Pornemby 风险急停被触发, 所有操作永久停止"
            if reason:
                msg += f" (原因: {reason})"
            msg += "."
            self.log.bind(log=True).error(msg)
            async with self.lock:
                self.alert_remaining = float("inf")

    async def check_pinned(self, message: Message):
        if message.service == MessageServiceType.PINNED_MESSAGE:
            return message.pinned_message
        elif (not message.text) and (not message.media) and (not message.service) and (not message.game):
            async for message in self.client.search_messages(message.chat.id, filter=MessagesFilter.PINNED):
                return message
        else:
            return None

    async def on_trigger(self, message: Message, key, reply):
        # 管理员回复水群消息: 永久停止, 若存在关键词即回复
        # 用户回复水群消息, 停止 3600 秒, 若存在关键词即回复
        if message.reply_to_message_id in pornemby_messager_mids.get(self.client.me.id, []):
            if await self.check_admin(message.chat, message.from_user):
                await self.set_alert(reason="管理员回复了水群消息")
            else:
                await self.set_alert(3600, reason="非管理员回复了水群消息")
            if self.check_keyword(message, self.alert_reply_keywords):
                if not self.check_keyword(message, self.alert_reply_except_keywords):
                    if (not self.last_reply) or (
                        self.last_reply < datetime.now() - timedelta(seconds=self.reply_interval)
                    ):
                        await asyncio.sleep(random.uniform(5, 15))
                        await message.reply(random.choice(self.reply_words))
                        self.last_reply = datetime.now()
            return

        # 置顶消息, 若不在列表中停止 3600 秒, 否则停止 86400 秒
        pinned = await self.check_pinned(message)
        if pinned:
            self.pin_checked = True
            keyword = self.check_keyword(pinned, self.user_alert_keywords + self.admin_alert_keywords)
            if keyword:
                await self.set_alert(86400, reason=f'有新消息被置顶, 且包含风险关键词: "{keyword}"')
            else:
                await self.set_alert(3600, reason="有新消息被置顶")
            return

        if not self.pin_checked:
            async for pinned in self.client.search_messages(message.chat.id, filter=MessagesFilter.PINNED):
                self.pin_checked = True
                keyword = self.check_keyword(pinned, self.user_alert_keywords + self.admin_alert_keywords)
                if keyword:
                    await self.set_alert(86400, reason=f'检查到现有置顶消息中包含风险关键词: "{keyword}"')
                    break

        # 管理员发送消息, 若不在列表中停止 3600 秒, 否则停止 86400 秒
        # 用户发送列表中消息, 停止 1800 秒
        if await self.check_admin(message.chat, message.from_user):
            keyword = self.check_keyword(message, self.user_alert_keywords + self.admin_alert_keywords)
            if keyword:
                await self.set_alert(86400, reason=f'管理员发送了消息, 且包含风险关键词: "{keyword}"')
            else:
                await self.set_alert(3600, reason="管理员发送了消息")
        else:
            keyword = self.check_keyword(message, self.user_alert_keywords)
            if keyword:
                await self.set_alert(1800, reason=f'非管理员发送了消息, 且包含风险关键词: "{keyword}"')



import asyncio
import random
import re
from dataclasses import dataclass
from datetime import date, datetime, time, timedelta
from pathlib import Path
from typing import Iterable, List, Union

import yaml
from dateutil import parser
from loguru import logger
from pyrogram.types import User
from pyrogram.enums import ChatMemberStatus
from pyrogram.errors import ChatWriteForbidden, RPCError, SlowmodeWait
from schema import Optional, Schema, SchemaError


from ...data import get_data
from ...var import debug
from ...utils import show_exception, truncate_str, distribute_numbers
from ..tele import ClientsSession
from ..link import Link

__ignore__ = True


@dataclass(eq=False)
class _MessageSchedule:
    """定义一个发送规划, 即在特定时间段内某些消息中的一个有一定几率被发送."""

    messages: Iterable[str]
    at: Union[Iterable[Union[str, time]], Union[str, time]] = ("0:00", "23:59")
    possibility: float = 1.0
    multiply: int = 1
    only: str = None


@dataclass(eq=False)
class MessageSchedule:
    """定义一个发送规划, 即在特定时间段内某些消息中的一个有一定几率被发送, 允许使用一个话术列表资源名作为基础配置."""

    spec: str = None
    messages: Iterable[str] = None
    at: Union[Iterable[Union[str, time]], Union[str, time]] = None
    possibility: float = None
    only: str = None
    multiply: int = None

    def to_message_schedule(self) -> _MessageSchedule:
        return _MessageSchedule(
            messages=self.messages,
            at=self.at or ("0:00", "23:59"),
            possibility=self.possibility or 1,
            multiply=self.multiply or 1,
            only=self.only,
        )


@dataclass(eq=False)
class MessagePlan:
    """定义一个发送计划, 即在某事件发送某个消息."""

    message: str
    at: datetime
    schedule: _MessageSchedule
    skip: bool = False


class Messager:
    """自动水群类."""

    name: str = None  # 水群器名称
    chat_name: str = None  # 群聊的名称
    default_messages: List[Union[str, MessageSchedule]] = []  # 默认的话术列表资源名
    additional_auth: List[str] = []  # 额外认证要求
    min_interval: int = None  # 预设两条消息间的最小间隔时间
    max_interval: int = None  # 预设两条消息间的最大间隔时间

    site_last_message_time = None
    site_lock = asyncio.Lock()

    def __init__(self, account, me: User = None, nofail=True, proxy=None, basedir=None, config: dict = None):
        """
        自动水群类.
        参数:
            account: 账号登录信息
            me: 当前用户
            nofail: 启用错误处理外壳, 当错误时报错但不退出
            basedir: 文件存储默认位置
            proxy: 代理配置
            config: 当前水群器的特定配置
        """
        self.account = account
        self.nofail = nofail
        self.proxy = proxy
        self.basedir = basedir
        self.config = config
        self.me = me

        self.min_interval = config.get(
            "min_interval", config.get("interval", self.min_interval or 60)
        )  # 两条消息间的最小间隔时间
        self.max_interval = config.get("max_interval", self.max_interval)  # 两条消息间的最大间隔时间
        self.log = logger.bind(scheme="telemessager", name=self.name, username=me.name)
        self.timeline: List[MessagePlan] = []  # 消息计划序列

    def parse_message_yaml(self, file):
        """解析话术文件."""
        with open(file, "r", encoding="utf-8") as f:
            data = yaml.safe_load(f)
        schema = Schema(
            {
                "messages": [str],
                Optional("at"): [str],
                Optional("possibility"): float,
                Optional("only"): str,
            }
        )
        schema.validate(data)
        at = data.get("at", ("9:00", "23:00"))
        assert len(at) == 2
        at = [parser.parse(t).time() for t in at]
        return _MessageSchedule(
            messages=data.get("messages"),
            at=at,
            possibility=data.get("possibility", 1.0),
            only=data.get("only", None),
        )

    def add(self, schedule: _MessageSchedule, use_multiply=False):
        """根据规划, 生成计划, 并增加到时间线."""
        start_time, end_time = schedule.at
        if isinstance(start_time, str):
            start_time = parser.parse(start_time).time()
        if isinstance(end_time, str):
            end_time = parser.parse(end_time).time()
        start_datetime = datetime.combine(date.today(), start_time or time(0, 0))
        end_datetime = datetime.combine(date.today(), end_time or time(23, 59, 59))
        if end_datetime < start_datetime:
            end_datetime += timedelta(days=1)
        start_timestamp = start_datetime.timestamp()
        end_timestamp = end_datetime.timestamp()
        num_plans = schedule.multiply if use_multiply else 1
        base = [mp.at.timestamp() for mp in self.timeline]
        timestamps = distribute_numbers(
            start_timestamp, end_timestamp, num_plans, self.min_interval, self.max_interval, base=base
        )
        mps = []
        ignored = num_plans - len(timestamps)
        if ignored:
            self.log.warning(f"发生错误: 部分发送计划 ({ignored}) 无法排入当日日程并被跳过, 请检查您的配置.")
            for _ in range(ignored):
                mps.append(
                    MessagePlan(
                        message=random.choice(schedule.messages),
                        at=end_datetime,
                        schedule=schedule,
                        skip=True,
                    )
                )
        for t in timestamps:
            at = datetime.fromtimestamp(t)
            if at < datetime.now():
                at += timedelta(days=1)
            skip = False
            if random.random() >= schedule.possibility:
                skip = True
            elif schedule.only:
                today = datetime.today()
                if schedule.only.startswith("weekday") and today.weekday() > 4:
                    skip = True
                if schedule.only.startswith("weekend") and today.weekday() < 5:
                    skip = True
            mps.append(
                MessagePlan(
                    message=random.choice(schedule.messages),
                    at=at,
                    schedule=schedule,
                    skip=skip,
                )
            )
        self.timeline.extend(mps)
        self.timeline = sorted(self.timeline, key=lambda x: x.at)
        return True

    async def get_spec_path(self, spec):
        """下载话术文件对应的本地或云端文件."""
        if not Path(spec).exists():
            return await get_data(self.basedir, spec, proxy=self.proxy, caller=f"{self.name}水群")
        else:
            return spec

    async def get_spec_schedule(self, spec_or_schedule):
        """解析话术文件对应的本地或云端文件."""
        if isinstance(spec_or_schedule, MessageSchedule):
            if spec_or_schedule.spec:
                file = await self.get_spec_path(spec_or_schedule.spec)
                if not file:
                    self.log.warning(f'话术文件 "{spec_or_schedule.spec}" 无法下载或不存在, 将被跳过.')
                    return None
                try:
                    base_schedule = self.parse_message_yaml(file)
                    # Merge base_schedule with spec_or_schedule
                    return _MessageSchedule(
                        messages=spec_or_schedule.messages or base_schedule.messages,
                        at=spec_or_schedule.at or base_schedule.at,
                        possibility=spec_or_schedule.possibility or base_schedule.possibility,
                        multiply=spec_or_schedule.multiply or base_schedule.multiply,
                        only=spec_or_schedule.only or base_schedule.only,
                    )
                except (OSError, yaml.YAMLError, SchemaError) as e:
                    self.log.warning(f'话术文件 "{spec_or_schedule.spec}" 错误, 将被跳过: {e}')
                    show_exception(e)
                    return None
            else:
                return spec_or_schedule.to_message_schedule()
        else:
            file = await self.get_spec_path(spec_or_schedule)
            if not file:
                self.log.warning(f'话术文件 "{spec_or_schedule}" 无法下载或不存在, 将被跳过.')
                return None
            try:
                return self.parse_message_yaml(file)
            except (OSError, yaml.YAMLError, SchemaError) as e:
                self.log.warning(f'话术文件 "{spec_or_schedule}" 错误, 将被跳过: {e}')
                show_exception(e)
                return None

    async def _start(self):
        """自动水群器的入口函数的错误处理外壳."""
        try:
            return await self.start()
        except asyncio.CancelledError:
            raise
        except Exception as e:
            if self.nofail:
                self.log.warning(f"发生错误, 自动水群器将停止.")
                show_exception(e, regular=False)
                return False
            else:
                raise

    async def start(self):
        """自动水群器的入口函数."""
        if self.additional_auth:
            async with ClientsSession([self.account], proxy=self.proxy, basedir=self.basedir) as clients:
                async for tg in clients:
                    for a in self.additional_auth:
                        if not await Link(tg).auth(a, log_func=self.log.info):
                            return False

        if self.max_interval and self.min_interval > self.max_interval:
            self.log.warning(f"发生错误: 最小间隔不应大于最大间隔, 自动水群将停止.")
            return False

        if not await self.init():
            self.log.warning(f"状态初始化失败, 自动水群将停止.")
            return False

        messages = self.config.get("messages", [])
        if not messages:
            messages = self.default_messages

        schedules = []
        for m in messages:
            if isinstance(m, MessageSchedule):
                schedule = await self.get_spec_schedule(m)
                if schedule:
                    schedules.append(schedule)
            else:
                match = re.match(r"(.*)\*\s?(\d+)", m)
                if match:
                    multiply = int(match.group(2))
                    spec = match.group(1).strip()
                else:
                    multiply = 1
                    spec = m
                schedule = await self.get_spec_schedule(spec)
                if schedule:
                    schedule.multiply = multiply
                    schedules.append(schedule)

        nmsgs = sum([s.multiply for s in schedules])
        self.log.info(f"共启用 {len(schedules)} 个消息规划, 发送 {nmsgs} 条消息.")
        for s in schedules:
            self.add(s, use_multiply=True)

        if self.timeline:
            last_valid_p = None
            while True:
                valid_p = [p for p in self.timeline if not p.skip]
                self.log.debug(f"时间线上当前有 {len(self.timeline)} 个消息计划, {len(valid_p)} 个有效.")
                if debug > 1:
                    self.log.debug(
                        "时间序列: "
                        + " ".join([p.at.strftime("%d%H%M%S") for p in sorted(valid_p, key=lambda x: x.at)])
                    )
                if valid_p:
                    next_valid_p = min(valid_p, key=lambda x: x.at)
                    if not next_valid_p == last_valid_p:
                        last_valid_p = next_valid_p
                        self.log.info(
                            f"下一次发送将在 [blue]{next_valid_p.at.strftime('%m-%d %H:%M:%S')}[/] 进行: {truncate_str(next_valid_p.message, 20)}."
                        )
                else:
                    self.log.info(f"下一次发送被跳过.")
                next_p = min(self.timeline, key=lambda x: x.at)
                self.log.debug(
                    f"下一次计划任务将在 [blue]{next_p.at.strftime('%m-%d %H:%M:%S')}[/] 进行 ({'跳过' if next_p.skip else '有效'})."
                )
                await asyncio.sleep((next_p.at - datetime.now()).total_seconds())
                if not next_p.skip:
                    await self.send(next_p.message)
                self.timeline.remove(next_p)
                self.add(next_p.schedule)

    async def init(self):
        """可重写的初始化函数, 返回 False 将视为初始化错误."""
        return True

    async def send(self, message):
        """自动水群器的发信器的入口函数."""
        if self.site_last_message_time:
            need_sec = random.randint(5, 10)
            while self.site_last_message_time > datetime.now() - timedelta(seconds=need_sec):
                await asyncio.sleep(1)
        async with ClientsSession([self.account], proxy=self.proxy, basedir=self.basedir) as clients:
            async for tg in clients:
                chat = await tg.get_chat(self.chat_name)
                self.log.bind(username=tg.me.name).info(f'向聊天 "{chat.name}" 发送: [gray50]{message}[/]')
                try:
                    msg = await tg.send_message(self.chat_name, message)
                except ChatWriteForbidden as e:
                    try:
                        chat = await tg.get_chat(self.chat_name)
                        member = await chat.get_member(tg.me.id)
                        if member.status == ChatMemberStatus.RESTRICTED:
                            if member.until_date:
                                delta = member.until_date - datetime.now()
                                secs = delta.total_seconds()
                                if secs < 2592000:
                                    self.log.warning(f"发送失败: 您已被禁言 {secs} 秒.")
                                else:
                                    self.log.warning(f"发送失败: 您已被永久禁言.")
                            else:
                                self.log.warning(f"发送失败: 您已被禁言.")
                        else:
                            self.log.warning(f"发送失败: 已全员禁言.")
                    except RPCError:
                        self.log.warning(f"发送失败: {e}.")
                except SlowmodeWait as e:
                    self.log.info(f"发送等待: Telegram 限制, 等待 {e.value} 秒.")
                except RPCError as e:
                    self.log.warning(f"发送失败: {e}.")
                except KeyError as e:
                    self.log.warning(f"发送失败, 您可能已被封禁.")
                    show_exception(e)
                except Exception as e:
                    self.log.warning(f"发送失败: {e}.")
                else:
                    async with self.site_lock:
                        self.__class__.site_last_message_time = datetime.now()
                    return msg



import asyncio
import random
from typing import Callable, Coroutine, List, Optional, Tuple, Union
import uuid
from io import BytesIO

import yaml
import tomli
from loguru import logger
from pyrogram import filters
from pyrogram.handlers import MessageHandler
from pyrogram.types import Message
from pyrogram.errors.exceptions.bad_request_400 import YouBlockedUser
from pyrogram.errors import FloodWait

from ..utils import async_partial, truncate_str
from .lock import super_ad_shown, super_ad_shown_lock, authed_services, authed_services_lock
from .tele import Client


class LinkError(Exception):
    pass


class Link:
    """云服务类, 用于认证和高级权限任务通讯."""

    bot = "embykeeper_auth_bot"

    def __init__(self, client: Client):
        self.client = client
        self.log = logger.bind(scheme="telelink", username=client.me.name)

    @property
    def instance(self):
        """当前设备识别码."""
        rd = random.Random()
        rd.seed(uuid.getnode())
        return uuid.UUID(int=rd.getrandbits(128))

    async def delete_messages(self, messages: List[Message]):
        """删除一系列消息."""

        async def delete(m: Message):
            try:
                await m.delete(revoke=True)
                text = m.text or m.caption or "图片或其他内容"
                text = truncate_str(text.replace("\n", ""), 30)
                self.log.debug(f"[gray50]删除了 API 消息记录: {text}[/]")
            except asyncio.CancelledError:
                pass

        return await asyncio.gather(*[delete(m) for m in messages])

    async def post(
        self,
        cmd,
        photo=None,
        file=None,
        condition: Callable = None,
        timeout: int = 20,
        retries=3,
        name: str = None,
        fail: bool = False,
    ) -> Tuple[Optional[str], Optional[str]]:
        """
        向机器人发送请求.
        参数:
            cmd: 命令字符串
            condition: 布尔或函数, 参数为响应 toml 的字典形式, 决定该响应是否为有效响应.
            timeout: 超时 (s)
            retries: 最大重试次数
            name: 请求名称, 用于用户提示
            fail: 当出现错误时抛出错误, 而非发送日志
        """
        if photo and file:
            raise ValueError("can not use both photo and file")

        for r in range(retries):
            try:
                await self.client.mute_chat(self.bot)
            except FloodWait:
                self.log.debug(f"[gray50]设置禁用提醒因访问超限而失败: {self.bot}[/]")
            future = asyncio.Future()
            handler = MessageHandler(
                async_partial(self._handler, cmd=cmd, future=future, condition=condition),
                filters.text & filters.bot & filters.user(self.bot),
            )
            await self.client.add_handler(handler, group=1)
            try:
                messages = []
                if photo:
                    messages.append(await self.client.send_photo(self.bot, photo, caption=cmd))
                elif file:
                    messages.append(await self.client.send_document(self.bot, file, caption=cmd))
                else:
                    messages.append(await self.client.send_message(self.bot, cmd))
                self.log.debug(f"[gray50]-> {cmd}[/]")
                results = await asyncio.wait_for(future, timeout=timeout)
            except asyncio.CancelledError:
                try:
                    await asyncio.wait_for(self.delete_messages(messages), 1.0)
                except asyncio.TimeoutError:
                    pass
                finally:
                    raise
            except asyncio.TimeoutError:
                await self.delete_messages(messages)
                if r + 1 < retries:
                    self.log.info(f"{name}超时 ({r + 1}/{retries}), 将在 3 秒后重试.")
                    await asyncio.sleep(3)
                    continue
                else:
                    msg = f"{name}超时 ({r + 1}/{retries})."
                    if fail:
                        raise LinkError(msg)
                    else:
                        self.log.warning(msg)
                        return None
            except YouBlockedUser:
                msg = "您在账户中禁用了用于 API 信息传递的 Bot: @embykeeper_auth_bot, 这将导致 embykeeper 无法运行, 请尝试取消禁用."
                if fail:
                    raise LinkError(msg)
                else:
                    self.log.error(msg)
                    return None
            else:
                await self.delete_messages(messages)
                status, errmsg = [results.get(p, None) for p in ("status", "errmsg")]
                if status == "error":
                    if fail:
                        raise LinkError(f"{errmsg}.")
                    else:
                        self.log.warning(f"{name}错误: {errmsg}.")
                        return False
                elif status == "ok":
                    return results
                else:
                    if fail:
                        raise LinkError("出现未知错误.")
                    else:
                        self.log.warning(f"{name}出现未知错误.")
                        return False
            finally:
                await self.client.remove_handler(handler, group=1)

    async def _handler(
        self,
        client: Client,
        message: Message,
        cmd: str,
        future: asyncio.Future,
        condition: Union[bool, Callable[..., Coroutine], Callable] = None,
    ):
        try:
            toml = tomli.loads(message.text)
        except tomli.TOMLDecodeError:
            self.delete_messages([message])
        else:
            try:
                if toml.get("command", None) == cmd:
                    if condition is None:
                        cond = True
                    elif asyncio.iscoroutinefunction(condition):
                        cond = await condition(toml)
                    elif callable(condition):
                        cond = condition(toml)
                    if cond:
                        future.set_result(toml)
                        await asyncio.sleep(0.5)
                        await self.delete_messages([message])
                        return
            except asyncio.CancelledError as e:
                try:
                    await asyncio.wait_for(self.delete_messages([message]), 1)
                except asyncio.TimeoutError:
                    pass
                finally:
                    future.set_exception(e)
            finally:
                message.continue_propagation()

    async def auth(self, service: str, log_func=None):
        """向机器人发送授权请求."""
        async with authed_services_lock:
            user_auth_cache = authed_services.get(self.client.me.id, {}).get(service, None)
            if user_auth_cache is not None:
                return user_auth_cache

            # No cache, perform auth
            if not log_func:
                result = await self.post(
                    f"/auth {service} {self.instance}", name=f"服务 {service.upper()} 认证"
                )
                authed_services.setdefault(self.client.me.id, {})[service] = bool(result)
                return bool(result)
            else:
                try:
                    await self.post(
                        f"/auth {service} {self.instance}",
                        name=f"服务 {service.upper()} 认证",
                        fail=True,
                    )
                except LinkError as e:
                    log_func(f"初始化错误: 使用 {service.upper()} 服务, 但{e}")
                    if "权限不足" in str(e):
                        await self._show_super_ad()
                    authed_services.setdefault(self.client.me.id, {})[service] = False
                    return False
                else:
                    authed_services.setdefault(self.client.me.id, {})[service] = True
                    return True

    async def _show_super_ad(self):
        async with super_ad_shown_lock:
            user_super_ad_shown = super_ad_shown.get(self.client.me.id, False)
            if not user_super_ad_shown:
                self.log.info("请访问 https://go.zetx.tech/eksuper 赞助项目以升级为高级用户, 尊享更多功能.")
                super_ad_shown[self.client.me.id] = True
                return True
            else:
                return False

    async def captcha(self, site: str, url: str = None):
        """向机器人发送验证码解析请求."""
        cmd = f"/captcha {self.instance} {site}"
        if url:
            cmd += f" {url}"
        results = await self.post(cmd, timeout=240, name="请求跳过验证码")
        if results:
            return results.get("token", None)
        else:
            return None

    async def captcha_url(self, site: str, url: str = None):
        """向机器人发送带验证码的远程网页解析请求."""
        cmd = f"/captcha {self.instance} {site}"
        if url:
            cmd += f" {url}"
        results = await self.post(cmd, timeout=240, name="请求跳过验证码")
        if results:
            return results.get("cf_clearance", None), results.get("result", None)
        else:
            return None, None

    async def captcha_emby(self, url: str):
        """向机器人发送带验证码的远程网页解析请求."""
        cmd = f"/captcha {self.instance} emby {url}"
        results = await self.post(cmd, timeout=240, name="请求跳过验证码")
        if results:
            return results.get("cf_clearance", None), results.get("proxy", None)
        else:
            return None, None

    async def pornemby_answer(self, question: str):
        """向机器人发送问题回答请求."""
        results = await self.post(
            f"/pornemby_answer {self.instance} {question}", timeout=20, name="请求问题回答"
        )
        if results:
            return results.get("answer", None), results.get("by", None)
        else:
            return None, None

    async def terminus_answer(self, question: str):
        """向机器人发送问题回答请求."""
        results = await self.post(
            f"/terminus_answer {self.instance} {question}", timeout=20, name="请求问题回答"
        )
        if results:
            return results.get("answer", None), results.get("by", None)
        else:
            return None, None

    async def gpt(self, prompt: str):
        """向机器人发送智能回答请求."""
        results = await self.post(f"/gpt {self.instance} {prompt}", timeout=20, name="请求智能回答")
        if results:
            return results.get("answer", None), results.get("by", None)
        else:
            return None, None

    async def visual(self, photo, options: List[str], question=None):
        """向机器人发送视觉问题解答请求."""
        cmd = f"/visual {self.instance} {'/'.join(options)}"
        if question:
            cmd += f" {question}"
        results = await self.post(cmd, photo=photo, timeout=20, name="请求视觉问题解答")
        if results:
            return results.get("answer", None), results.get("by", None)
        else:
            return None, None

    async def ocr(self, photo):
        """向机器人发送 OCR 解答请求."""
        cmd = f"/ocr {self.instance}"
        results = await self.post(cmd, photo=photo, timeout=20, name="请求验证码解答")
        if results:
            return results.get("answer", None)
        else:
            return None

    async def send_log(self, message):
        """向机器人发送日志记录请求."""
        results = await self.post(f"/log {self.instance} {message}", name="发送日志到 Telegram ")
        return bool(results)

    async def send_msg(self, message):
        """向机器人发送即时日志记录请求."""
        results = await self.post(f"/msg {self.instance} {message}", name="发送即时日志到 Telegram ")
        return bool(results)

    async def infer_msg(self, data):
        """向机器人发送话术推测记录请求."""
        bio = BytesIO()
        bio.write(yaml.dump(data, allow_unicode=True).encode("utf-8"))
        bio.seek(0)
        bio.name = "data.yaml"

        results = await self.post(f"/infer_msg {self.instance}", file=bio, name="发送话术推测请求")
        if results:
            return results.get("answer", None), results.get("by", None)
        else:
            return None, None



from ._base import BotCheckin

__ignore__ = True


class ZhipianCheckin(BotCheckin):
    name = "纸片"
    bot_username = "zhipianbot"
    bot_checkin_cmd = "/qd"
    additional_auth = ["prime"]
    checked_retries = 6



from .pornemby import PornembyMessager

__ignore__ = True


class TestPornembyMessager(PornembyMessager):
    name = "Pornemby 水群测试"
    chat_name = "api_group"
    default_messages = ["pornemby-common-wl@latest.yaml * 1000"]



import asyncio

from pyrogram.types import Message
from thefuzz import fuzz

from ._base import AnswerBotCheckin

__ignore__ = True


class LJYYCheckin(AnswerBotCheckin):
    ocr = "uchars4@v1"

    name = "垃圾影音"
    bot_username = "zckllflbot"
    bot_checkin_cmd = ["/stop", "/cz"]
    bot_use_history = 20
    bot_text_ignore = ["下列选项", "退出流程"]
    bot_checkin_button_pat = r"\w\s\w\s\w\s\w"
    additional_auth = ["prime"]

    async def message_handler(self, client, message: Message):
        if message.text and "请选择操作项" in message.text and message.reply_markup:
            keys = [k.text for r in message.reply_markup.inline_keyboard for k in r]
            for k in keys:
                if "签到" in k:
                    async with self.client.catch_reply(self.bot_username) as f:
                        try:
                            await message.click(k)
                        except TimeoutError:
                            pass
                        try:
                            await asyncio.wait_for(f, 10)
                        except asyncio.TimeoutError:
                            self.log.warning(f"签到失败: 点击签到无响应.")
                            await self.fail()
            else:
                self.log.warning(f"签到失败: 账户错误.")
                return await self.fail()
        await super().message_handler(client, message)

    async def on_captcha(self, message: Message, captcha: str):
        async with self.operable:
            if not self.message:
                await self.operable.wait()
            match = [(k, fuzz.ratio(k, captcha)) for k in self.get_keys(self.message)]
            max_k, max_r = max(match, key=lambda x: x[1])
            self.log.info(f"识别字符: {captcha}, 选择字符: {max_k}")
            await self.message.click(max_k)



# 该文件用于同机器人 Messager, Monitor 和 Bots 之间的异步锁和通讯

import asyncio

from cachetools import TTLCache

ocrs = TTLCache(maxsize=1024, ttl=3600)  # spec: (DdddOcr, bool)
ocrs_lock = asyncio.Lock()

misty_monitors = {}  # uid: MistyMonitor
misty_locks = {}  # uid: lock

pornemby_nohp = {}  # uid: date
pornemby_messager_enabled = {}  # uid: bool
pornemby_alert = {}  # uid: bool
pornemby_messager_mids = {}  # uid: list(mid)

super_ad_shown = {}  # uid: bool
super_ad_shown_lock = asyncio.Lock()

authed_services = {}  # uid: {service: bool}
authed_services_lock = asyncio.Lock()



from pyrogram.types import Message

from ._base import BotCheckin

__ignore__ = True


class SingularityCheckin(BotCheckin):
    name = "Singularity"
    bot_username = "Singularity_Emby_Bot"
    bot_checkin_cmd = "/start"
    bot_captcha_len = None
    bot_checkin_caption_pat = "请输入验证码"

    async def message_handler(self, client, message: Message):
        if message.caption and "欢迎使用" in message.caption and message.reply_markup:
            keys = [k.text for r in message.reply_markup.inline_keyboard for k in r]
            for k in keys:
                if "签到" in k:
                    await message.click(k)
                    return
            else:
                self.log.warning(f"签到失败: 账户错误.")
                return await self.fail()
        await super().message_handler(client, message)



from ._base import BotCheckin

from pyrogram.types import Message
from pyrogram.raw.types.messages import BotCallbackAnswer

__ignore__ = True


class MiniSGKCheckin(BotCheckin):
    name = "迷你世界社工库"
    bot_username = "mnsjsgkbot"
    bot_checkin_cmd = "/sign"
    additional_auth = ["prime"]

    async def message_handler(self, client, message: Message):
        if message.reply_markup and message.reply_markup.inline_keyboard:
            keys = [k.text for r in message.reply_markup.inline_keyboard for k in r]
            for k in keys:
                if "签到" in k:
                    answer: BotCallbackAnswer = await message.click(k)
                    await self.on_text(Message(id=0), answer.message)
                    return
            else:
                self.log.warning(f"签到失败: 账户错误.")
                return await self.fail()
